#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
from gi.repository import Gtk
from gi.repository import GdkPixbuf
	
class Window(Gtk.Window):
	def __init__(self,title):
		super(Window,self).__init__()
		self.set_title(title)
		
	
	def connect_close_function(self,function):
		self.connect("delete-event",function)
	
	def connect_menubar(self,menubar):
		self.add_accel_group(menubar.get_accel_group())
	
	def connect_configure_event_handler(self,function):
		self.connect("configure-event",function)

	def set_taskbar_icon(self,file_path):
		pixbuf = GdkPixbuf.Pixbuf.new_from_file(file_path)
		self.set_icon(pixbuf)
		

	#add()
	#get_size()



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2014-2015 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import enchant

dictionary_language_dict = {"eng" : "en","afr" : "af","am" : "am",
		"ara" : "ar","ara" : "ar","aze":"az","bel":"be","bul" : "bg",

		#Recently added
		"ben" : "bn","br" : "br","cat" : "ca","ces" : "cs","cy" : "cy",
		"dan" : "da","ger" : "de","deu":"du","deu-frak":"du","ell":"el",
		"epo":"eo","eq":"en","eus":"eu-es","frk":"fk","frm":"fr-classical",
		"grc":"el","ind":"id","isl":"is","ita-old" : "it","jpn":"jn",
		"kan":"kn","kor":"ko","mkd":"md","mlt":"mt","msa":"ma","nld":"nl",
		"nor":"no","spa-old":"es","sqi":"si","srp":"sh","swa":"sw","tha":"th",
		"tur":"tr","vie":"vi","glg" : "gl",

		"ger" : "de","ell" : "el","eo" : "eo","spa" : "es","est" : "et",
		"eu" : "eu","fa" : "fa","fin" : "fi","fo" : "fo","fra" : "fr",
		"ga" : "ga","gl" : "gl","gu" : "gu","heb" : "he","hin" : "hi",
		"hrv" : "hr","hsb" : "hsb","hun" : "hu","hy" : "hy","id" : "id",
		"is" : "is","ita" : "it","kk" : "kk","kn" : "kn","ku" : "ku",
		"lit" : "lt","lav" : "lv","mal" : "ml ","mr" : "mr ","dut" : "nl",
		"no" : "no","nr" : "nr","ns" : "ns ","or" : "or ","pa" : "pa ",
		"pol" : "pl ","por" : "pt","por" : "pt","por" : "pt","ron" : "ro",
		"rus" : "ru ","slk" : "sk","slv" : "sl","ss" : "ss","st" : "st",
		"swe" : "sv","tam" : "ta","tel" : "te","tl" : "tl","tn" : "tn",
		"ts" : "ts","ukr" : "uk","uz" : "uz","xh" : "xh","zu" : "zu" }

class Dict(enchant.Dict):
	def __init__(self,language="en"):
		super(Dict,self).__init__(language)
	
	#list_languages()
	#suggest(word)
	#check(word)





#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
from gi.repository import Pango

class TextView(Gtk.TextView):
	AT_START=0
	AT_CURSOR=1
	AT_END=2
	
	def __init__(self):
		super(TextView, self).__init__()
		self.set_wrap_mode(Gtk.WrapMode.WORD)
		buffer = self.get_buffer()
		self.highlight_tag = buffer.create_tag('Reading')		
	
	def connect_insert(self,function):
		buffer = self.get_buffer()
		buffer.connect("insert-text",lambda x, y,z,a  : function())

	def connect_delete(self,function):
		buffer = self.get_buffer()
		buffer.connect("delete-range",lambda x, y,z  : function())

	def set_text(self,text):
		buffer = self.get_buffer()
		buffer.set_text(text);
	
	def get_text(self):
		buffer = self.get_buffer()
		start,end = buffer.get_bounds()
		text = buffer.get_text(start,end,False)
		return text

	# For Text Cleaner
	def get_text_from_cursor_to_end(self):
		buffer = self.get_buffer()
		end = buffer.get_end_iter()
		mark = buffer.get_insert()
		start = buffer.get_iter_at_mark(mark)
		text = buffer.get_text(start,end,False)
		return text

	def delete_text_from_cursor_to_end(self):
		buffer = self.get_buffer()
		end = buffer.get_end_iter()
		mark = buffer.get_insert()
		start = buffer.get_iter_at_mark(mark)
		buffer.delete(start,end)
	
	def insert_text(self,text,position,place_cursor=False):
		buffer = self.get_buffer()
		if (position == TextView.AT_START):
			iter = buffer.get_start_iter()
			buffer.insert(iter,text)
			
		elif position == TextView.AT_CURSOR:
			buffer.insert_at_cursor(text)
		else:
			iter = buffer.get_end_iter()
			buffer.insert(iter,text)
	
	def set_highlight_font(self,highlight_font):
		self.highlight_tag.set_property('font',highlight_font)

	def set_highlight_color(self,highlight_color):
		self.highlight_tag.set_property('foreground',
			Gdk.color_parse(highlight_color).to_string())

	def set_highlight_background(self,background_highlight_color):
		self.highlight_tag.set_property('background',
			Gdk.color_parse(background_highlight_color).to_string())
	
		
	def set_font(self,font_discription):
		pangoFont = Pango.FontDescription(font_discription)
		self.modify_font(pangoFont)
	
	def set_font_color(self,font_color):
		self.modify_fg(Gtk.StateFlags.NORMAL, Gdk.color_parse(font_color))
		
	def set_background_color(self,background_color):
		self.modify_bg(Gtk.StateFlags.NORMAL, Gdk.color_parse(background_color))
	
	def get_cursor_line_number(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		offset = buffer.get_iter_at_mark(insert_mark)
		return offset.get_line()
	
	def get_line_count(self):
		buffer = self.get_buffer()
		return buffer.get_line_count()

	def move_cursor_to_line(self,line_number):
		buffer = self.get_buffer()
		iter = buffer.get_iter_at_line(line_number)
		buffer.place_cursor(iter)
		self.scroll_to_iter(iter, 0.0,False,0.0,0.0)
	
	def get_modified(self):
		buffer = self.get_buffer()
		return buffer.get_modified()
	
	def set_modified(self,value):
		buffer = self.get_buffer()
		buffer.set_modified(value)
	
	def has_selection(self):
		buffer = self.get_buffer()
		return buffer.get_has_selection()

	def get_selected_text(self):
		buffer = self.get_buffer()
		start,end = buffer.get_selection_bounds()
		return buffer.get_text(start,end,0)
	
	def delete_all_text(self):
		buffer = self.get_buffer()
		start, end = buffer.get_bounds()
		buffer.delete(start, end)
	
	# Placing a bookmark
	def get_mark_at_line(self,line):
		buffer = self.get_buffer()
		iter = buffer.get_iter_at_line(line)
		return buffer.create_mark(None,iter,True)
		
	# Moving to existing bookmark
	def move_cursor_to_mark(self,mark):
		buffer = self.get_buffer()
		iter = buffer.get_iter_at_mark(mark)
		buffer.place_cursor(iter)
		self.scroll_to_iter(iter, 0.0,False,0.0,0.0)
	
	# For saving Bookmark with line number
	def get_line_number_of_mark(self,mark):
		buffer = self.get_buffer()
		iter = buffer.get_iter_at_mark(mark)
		return iter.get_line()
		
	def get_cursor_mark(self):
		buffer = self.get_buffer()
		return buffer.get_insert()
	
	# For autofilling new bookmark name
	def get_current_line_text(self):
		buffer = self.get_buffer()
		mark = buffer.get_insert()
		start_iter = buffer.get_iter_at_mark(mark)
		end_iter = start_iter.copy()
		start_iter.backward_sentence_start()
		end_iter.forward_to_line_end()
		return buffer.get_text(start_iter,end_iter,True)		
	
	# For highlighting bookmark position and go-to-line
	def highlights_cursor_line(self):
		buffer = self.get_buffer()
		self.remove_all_highlights()
		mark = buffer.get_insert()
		start_iter = buffer.get_iter_at_mark(mark)
		end_iter = start_iter.copy()
		end_iter.forward_to_line_end()
		self.scroll_to_iter(start_iter, 0.0,False,0.0,0.0)
		buffer.apply_tag(self.highlight_tag, start_iter, end_iter)
	
	#Find , Find and Replace , Spell check , TextReader
	

	
	def remove_all_highlights(self):
		buffer = self.get_buffer()
		start = buffer.get_start_iter()
		end = buffer.get_end_iter()
		buffer.remove_tag(self.highlight_tag, start, end)
		
		
			
	def replace_last_word(self,replace_word):
		buffer = self.get_buffer()
		buffer.delete(self.start_iter, self.end_iter)
		buffer.insert(self.start_iter," "+replace_word)
		self.start_iter = self.end_iter.copy()

	def get_next_word(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		self.start_iter = buffer.get_iter_at_mark(insert_mark)
		self.end_iter = self.start_iter.copy()
		iter0 = self.start_iter.copy()
		iter0.backward_word_start()
		self.remove_all_highlights()
		self.end_iter.forward_word_end()
		buffer.apply_tag(self.highlight_tag, self.start_iter, self.end_iter)
		buffer.place_cursor(self.end_iter)
		return buffer.get_text(self.start_iter,self.end_iter,0)			

	def get_previous_word(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		self.end_iter = buffer.get_iter_at_mark(insert_mark)
		self.start_iter = self.end_iter.copy()		
		iter0 = self.end_iter.copy()
		iter0.forward_word_end()
		self.remove_all_highlights()		
		self.start_iter.backward_word_start ()
		buffer.apply_tag(self.highlight_tag, self.start_iter, self.end_iter)
		buffer.place_cursor(self.start_iter)
		return buffer.get_text(self.start_iter,self.end_iter,0)

	def get_context_text(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		iter1 = buffer.get_iter_at_mark(insert_mark)
		iter2 = iter1.copy()
		iter1.backward_sentence_start()
		iter2.forward_sentence_end()
		return buffer.get_text(iter1,iter2,0)
	
	def delete_last_word(self):
		buffer = self.get_buffer()
		buffer.delete(self.start_iter,self.end_iter)
	
	def get_next_sentence(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		iter1 = buffer.get_iter_at_mark(insert_mark)
		iter2 = iter1.copy()
		end = buffer.get_end_iter()
		self.remove_all_highlights()
		lineString = ""
		# Get text word by word till a full stop followed by a space, CR or LF found.
		# If no full stop found, stop at minimum of 1000 more chars

		# Full Stop for English(u002e),  Devanagari(u0964), Sinhala(u0df4),
		# Chinese/Japanese(u3002), Arabic(u06d4)

		# Space - 0020, Carriage return(CR) - 000d , line feed (LF) - 000a

		while( '\u002e\u0020' not in lineString \
		and '\u002e\u000d' not in lineString \
		and '\u002e\u000a' not in lineString \
		and '\u0964\u0020' not in lineString \
		and '\u0964\u000d' not in lineString \
		and '\u0964\u000a' not in lineString \
		and '\u0df4\u0020' not in lineString \
		and '\u0df4\u000d' not in lineString \
		and '\u0df4\u000a' not in lineString \
		and '\u3002\u0020' not in lineString \
		and '\u3002\u000a' not in lineString \
		and '\u3002\u000d' not in lineString \
		and '\u06d4\u0020' not in lineString \
		and '\u06d4\u000a' not in lineString \
		and '\u06d4\u000d' not in lineString \
		and len(lineString) < 1000 and not end.equal(iter2)):
			iter2.forward_char()
			lineString = buffer.get_text(iter1,iter2,0)

		buffer.apply_tag(self.highlight_tag, iter1, iter2)
		buffer.place_cursor(iter2)
		return buffer.get_text(iter1,iter2,0)
	
	def is_cursor_at_end(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		iter1 = buffer.get_iter_at_mark(insert_mark)
		end = buffer.get_end_iter()
		if(end.equal(iter1)):
			return 1;
		else:
			return 0;

	def is_cursor_at_start(self):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		iter1 = buffer.get_iter_at_mark(insert_mark)
		start = buffer.get_start_iter()
		if(start.equal(iter1)):
			return 1;
		else:
			return 0;
	
	# For Find/Find and Replace
	
	def move_forward_to_word(self,word):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		self.start_iter = buffer.get_iter_at_mark(insert_mark)
		result = self.start_iter.forward_search(word, 0, None)
		if (result):
			self.start_iter, self.end_iter = result
			buffer.place_cursor(self.end_iter)
			self.remove_all_highlights()
			buffer.apply_tag(self.highlight_tag, self.start_iter, self.end_iter)
			self.scroll_to_iter(self.start_iter,00,False,False,False)
			return True
		return False

	def move_backward_to_word(self,word):
		buffer = self.get_buffer()
		insert_mark = buffer.get_insert()
		self.start_iter = buffer.get_iter_at_mark(insert_mark)
		self.end_iter = self.start_iter.copy()
		result = self.start_iter.backward_search(word, 0, None)
		if (result):
			self.start_iter, self.end_iter = result
			buffer.place_cursor(self.start_iter)
			self.remove_all_highlights()
			buffer.apply_tag(self.highlight_tag, self.start_iter, self.end_iter)
			self.scroll_to_iter(self.start_iter,00,False,False,False)
			return True
		return False




#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2015 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import itertools

def get_list_of_mixed_case_combinations(list_items):
	return list(itertools.chain.from_iterable([[''.join(a) for a in itertools.product(*zip(s.upper(), s.lower()))] for s in list_items]))

user_home_path = os.environ['HOME']

config_dir = user_home_path+"/.lios"

tmp_dir = "/tmp/Lios/"

bookmarks_dir = config_dir+"/bookmarks/"

local_text_cleaner_list_file_path = config_dir+"/text_cleaner_list.text"

default_text_cleaner_list_file_path = "/usr/share/lios/text_cleaner_list.text"

preferences_file_path = config_dir+"/preferences.cfg"

recent_file_path = config_dir+"/recent.text"

supported_image_formats = get_list_of_mixed_case_combinations(["png","pnm","jpg","jpeg","tif","tiff","bmp","pbm","ppm"])

supported_text_formats = get_list_of_mixed_case_combinations(["txt","text"])

supported_pdf_formats = get_list_of_mixed_case_combinations(["pdf"])

version = "2.7.1"

logo_file = "/usr/share/lios/lios.png"

icon_dir = "/usr/share/lios/icons/"

readme_file = "/usr/share/lios/readme.text"

default_text_cleaner_list_file_path = "/usr/share/lios/text_cleaner_list.text"

app_name = "Linux-intelligent-ocr-solution"

source_link = "https://gitlab.com/Nalin-x-Linux/lios-3"

home_page_link = "http://sourceforge.net/projects/lios/"

video_tutorials_link = "https://www.youtube.com/playlist?list=PLn29o8rxtRe1zS1r2-yGm1DNMOZCgdU0i"

major_character_encodings_list = [ 'us_ascii', 'utf-8', 'iso_8859_1','latin1',
 'iso_8859_2', 'iso_8859_7', 'iso_8859_9', 'iso_8859_15', 'eucjp', 'euckr',
 'gb2312_80', 'gb2312_1980', 'windows_1251', 'windows_1252', 'windows_1253',
 'windows_1254', 'windows_1255', 'windows_1256', 'windows_1257', 'windows_1258',
 'shiftjis', 'windows_1256', 'big5_hkscs', 'big5_tw', 'tis620']



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2015 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import subprocess

class text_to_audio_converter:
	def __init__(self,text="Blank",volume=100,voice="english",split_time=5,pitch=50,speed=170):
		self.text = text
		self.set_volume(volume)
		self.set_voice(voice)
		self.set_split_time(split_time)
		self.set_pitch(pitch)
		self.set_speed(speed)	
		
	
	def list_voices():
		voice_list = []
		output = subprocess.getoutput("espeak --voices")
		for line in output.split("\n"):
			if(line.split()[3] != "VoiceName"):
				voice_list.append(line.split()[3])
		return voice_list

		
	def get_volume(self):
		return self.volume

	def get_voice(self):
		return self.voice

	def get_split_time(self):
		return self.split_time
				
	def get_pitch(self):
		return self.pitch

	def get_speed(self):
		return self.speed


	def set_volume(self,value):
		if ( 0 <= value and value <= 200):
			self.volume = value
		else:
			self.volume = 100
			return False;

	def set_voice(self,value):
		if (value in text_to_audio_converter.list_voices()):
			self.voice = value
			return True;
		else:
			self.voice = "english"
			return False;
			

	def set_split_time(self,value):
		self.split_time  = value
				
	def set_pitch(self,value):
		if ( 0 <= value and value <= 100):
			self.pitch = value
			return True
		else:
			self.pitch = 50
			return False;

	def set_speed(self,value):
		if ( 100 <= value and value <= 450):
			self.speed = value
			return True
		else:
			self.speed = 170
			return False;
		
	def record_to_wave(self,output_file_name):
		to_convert = open("tmp.txt",'w')
		to_convert.write(self.text)
		to_convert.close()
		os.system('espeak -a %s -v %s -f tmp.txt -w %s.wav --split=%s -p %s -s %s' % (self.volume,self.voice,output_file_name,self.split_time,self.pitch,self.speed))

	def record_to_mp3(self,output_file_name):
		to_convert = open("tmp.txt",'w')
		to_convert.write(self.text)
		to_convert.close()
		os.system("rm -rf /tmp/lios_audio")
		os.system("mkdir /tmp/lios_audio")
		os.system('espeak -a %s -v %s -f tmp.txt -w /tmp/lios_audio/%s.wav --split=%s -p %s -s %s'%(self.volume,self.voice,output_file_name.split("/")[-1],self.split_time,self.pitch,self.speed))
		cmd = 'for f in /tmp/lios_audio/*.wav; do filename=$(basename "$f");name="${filename%.*}"; echo $name; lame --vbr-new -V 3  "$f" '+"/".join(output_file_name.split("/")[:-1])+'/$name.mp3; done;'
		os.system(cmd)


if __name__ == "__main__":
	ob = text_to_audio_converter("This is a sample of text in english")
	ob.record_to_wave("~/Hello")
	



#!/usr/bin/python3 
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import subprocess 
from lios.ocr.ocr_engine_base import OcrEngineBase

class OcrEngineOcrad(OcrEngineBase):
	name = "Ocrad"
	
	def __init__(self,language=None):
		self.set_language(language)

	def is_available():
		if ("/bin/ocrad" in subprocess.getoutput("whereis ocrad")):
			return True
		else:
			return False
			
	def ocr_image_to_text(self,file_name):
		os.system("convert {} /tmp/{}_for_ocr.pnm".format(file_name,file_name.split("/")[-1]))
		os.system("ocrad /tmp/{0}_for_ocr.pnm -l -c iso-8859-9 -o /tmp/{0}_output.txt".format(file_name.split("/")[-1]))
		os.remove("/tmp/{0}_for_ocr.pnm".format(file_name.split("/")[-1]))
		try:
			with open("/tmp/{0}_output.txt".format(file_name.split("/")[-1]),encoding="iso-8859-9") as file:
				text = file.read().strip()
				os.remove("/tmp/{0}_output.txt".format(file_name.split("/")[-1]))
				return text
		except:
			return ""

	def cancel():
		os.system("pkill convert")
		os.system("pkill ocrad")
		
	
	def get_available_languages():
		langs = ["eng"]
		return langs



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################


def detect_overlap(list,new_start_x,new_start_y,new_end_x,new_end_y):
	for item in list:
		start_x = item[0]
		start_y = item[1]
		finish_x = item[2]+item[0]
		finish_y = item[3]+item[1]
		if (((start_x <= new_start_x <= finish_x or start_x <= new_end_x <= finish_x) and
		 (start_y <= new_start_y <= finish_y or start_y <= new_end_y <= finish_y)) or 
		 # The rectangle maybe inside the new rectangle 
		 ((new_start_x <= start_x <= new_end_x and new_start_y <= start_y <= new_end_y) or 
		 ( new_start_x <= finish_x <= new_end_x and new_start_y <= finish_y <= new_end_y))):
			return (start_x,start_y,finish_x,finish_y)
	return False


def detect_out_of_range(x,y,max_width,max_height):
	if (x >= max_width or y >= max_height or x <= 0 or y<=0 ):
		return True
	else:
		return False


def order_rectangle(start_x,start_y,finish_x,finish_y):
	#Swap coordinate if selected in reverse direction
	if(start_x >= finish_x):
		start_x,finish_x = finish_x,start_x
	if(start_y >= finish_y):
		start_y,finish_y = finish_y,start_y
	return start_x,start_y,finish_x,finish_y
		

def is_overlapping(rs,index,a,b,c,d):
	nrs = [];
	for i in range(0,len(rs)):
		if(index != i):
			nrs.append(rs[i]);
	if(detect_overlap(nrs,a,b,a+c,b+d)):
		return True
	else:
		return False
	
	
		
def get_point_type(x,y,area_x_start, area_y_start,width,height,rs):

	# If pointer is at the edge of drawing area then mouse pointer must become normal
	if (x < area_x_start+10 or
	(area_x_start+width-20) <= x <= area_x_start+width or
	y < area_y_start+10 or
	(area_y_start+height-20) <= y <= area_y_start+height):
		return (1,-1,0)

	for i in range(0,len(rs)):
		scope_scale_y = (rs[i][3])/10
		scope_scale_x = (rs[i][2])/10
		
		#Left
		if(rs[i][0] <= x+scope_scale_x and
			rs[i][0] >= x-scope_scale_x and 
			rs[i][1] <= y and
			rs[i][3]+rs[i][1] >= y ):
				return (2,i,4)
						

		#Right 
		if(rs[i][0]+rs[i][2] <= x+scope_scale_x and
			rs[i][0]+rs[i][2] >= x-scope_scale_x and
			rs[i][1] <= y and
			rs[i][3]+rs[i][1] >= y ):
				return (2,i,6)

		#Top
		if(rs[i][1] <= y+scope_scale_y and
			rs[i][1] >= y-scope_scale_y and
			rs[i][0] <= x and
			rs[i][2]+rs[i][0] >= x ):
				return (2,i,2)

		#Bottom
		if(rs[i][3]+rs[i][1] <= y+scope_scale_y and
			rs[i][3]+rs[i][1] >= y-scope_scale_y and
			rs[i][0] <= x and
			rs[i][2]+rs[i][0] >= x ):
				return (2,i,8)

		#Top Left 
		if(rs[i][0] <= x+scope_scale_x and
			rs[i][0] >= x-scope_scale_x and
			rs[i][1] <= y+scope_scale_y and
			rs[i][1] >= y-scope_scale_y ):
				return (2,i,1)


		#Bottom Left 
		if(rs[i][0] <= x+scope_scale_x and
			rs[i][0] >= x-scope_scale_x and
			rs[i][3]+rs[i][1] <= y+scope_scale_y and
			rs[i][3]+rs[i][1] >= y-scope_scale_y):
				return (2,i,7)

		#Top Right 
		if(rs[i][0]+rs[i][2] <= x+scope_scale_x and
			rs[i][0]+rs[i][2] >= x-scope_scale_x and
			rs[i][1] <= y+scope_scale_y and
			rs[i][1] >= y-scope_scale_y ):
				return (2,i,3)


		#Bottom Right 
		if(rs[i][0]+rs[i][2] <= x+scope_scale_x and
			rs[i][0]+rs[i][2] >= x-scope_scale_x and
			rs[i][3]+rs[i][1] <= y+scope_scale_y and
			rs[i][3]+rs[i][1] >= y-scope_scale_y):
				return (2,i,9)
							

		#Moving
		if(x >= rs[i][0]+scope_scale_x and
			y >= rs[i][1]+scope_scale_y and
			x <= rs[i][0]+rs[i][2]-scope_scale_x and
			y <= rs[i][1]+rs[i][3]-scope_scale_y):
				return (3,i,5)

	return (1,-1,0)


# Used to find the index of new box inside the box list 
# The box list is arranged in top-left to bottom-right order
def find_index_for_new_box(new_s_x,new_s_y,new_e_x,new_e_y,rl):

	length = len(rl);
	index = length-1;

	min_x_dist = 99999;
	min_y_dist = 99999;

	for i in range (0,length):
		cur_s_x = rl[i][0]
		cur_s_y = rl[i][1]
		cur_e_x = cur_s_x+rl[i][2]
		cur_e_y = cur_s_y+rl[i][3]

		# Checking it's on the right side or not
		if(cur_s_x < new_s_x):
			# Checking new top and new bottom are inside the current one's y axis
			if(cur_s_y <= new_s_y <= cur_e_y or
			new_s_y <= cur_s_y <= new_e_y or
			cur_s_y <= new_e_y <= cur_e_y or
			new_s_y <= cur_e_y <= new_e_y):
				dist = new_s_x - cur_s_x;
				# The new box is residing on right side so
				# if cur box having minimum x distance to new then it's more close
				if ( dist < min_x_dist):
					min_x_dist = dist;
					index = i+1;

		#if nothing in parallel to the right then new box will be drawed in first from left
		if (min_x_dist == 99999):
			if(cur_e_y <= new_s_y):
				# Checking the vertical distance bitween new box and cur box
				# if it's very low then find the right most parallel one
				dist1 = new_s_y - cur_s_y;
				dist2 = new_s_y - cur_e_y;
				dist = (dist1+dist2)/2;
				if (dist < min_y_dist):
					min_y_dist = dist
					# Checking if any box in parallel to right side
					j = i+1;
					index = i+1;

					#cur_e_y >= rl[j][1] is used to skip symbols like single quotes
					while(j < length and cur_s_x < rl[j][0] and (cur_s_y <= rl[j][1] <= cur_e_y or
						rl[j][1] <= cur_s_y <= (rl[j][1]+rl[j][3]) or
						cur_s_y <= (rl[j][1]+rl[j][3]) <= cur_e_y or
						rl[j][1] <= cur_e_y <= (rl[j][1]+rl[j][3]) or cur_e_y >= rl[j][1] )):
							j = j + 1;
							index = j;
							# Stop if end reached
							if(j == length):
								break;
	return index;



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk	
from lios.ui.gtk import icon
from lios import macros
		

class Grid(Gtk.Grid):
	BOTTOM = Gtk.PositionType.BOTTOM
	RIGHT = Gtk.PositionType.RIGHT
	NEW_ROW = 1;
	VEXPAND = 1;
	HEXPAND = 1;
	NO_VEXPAND = 0;
	NO_HEXPAND = 0;
	ALIGN_END = Gtk.Align.END
	ALIGN_START = Gtk.Align.START
	
	def __init__(self):
		super(Grid,self).__init__()
		self.x = 0;
		self.y = 0;
		self.x_pad = 0;
		self.x_pad_count = 0;
		self.y_pad = 0;
		self.y_pad_count = 0;		
	
	def add_widgets(self,list):
		for item in list:
			if (item == Grid.NEW_ROW):
				self.__add_new_row()
			else:
				self.__add_widget(*item)
	
	#attach(widget, left, top, width, height)
	#attach_next_to(child, sibling, side, width, height)
	def __add_widget(self,child, width, height,hexpand=True,vexpand=True,halign=None,valign=None):
		child.set_hexpand(hexpand)
		child.set_vexpand(vexpand)
		if(halign):
			child.set_halign(halign)
		if(valign):
			child.set_valign(valign)
		
		if (self.x_pad_count > 1 and self.x < self.x_pad):
			self.x = self.x_pad
			self.x_pad_count = self.x_pad_count - 1 
			
		self.attach(child, self.x, self.y, width, height)
		if (height > 1):
			self.x_pad = width;
			self.x_pad_count = height
		
		self.x = self.x + width

	def __add_new_row(self):
		self.x = 0
		self.y = self.y + 1
		
		 
	
class ScrollBox(Gtk.ScrolledWindow):
	def __init__(self):
		super(ScrollBox,self).__init__()
		self.set_border_width(2)

	def scroll(self,h_value,v_value):
		adj = self.get_vadjustment()
		adj.set_value(v_value)
		adj = self.get_hadjustment()
		adj.set_value(h_value)

	def get_size_on_screen(self):
		alc = self.get_allocation()
		return (alc.width,alc.height)

	def get_current_start_points(self):
		hadj = self.get_hadjustment()
		start_x = hadj.get_value()
		vadj = self.get_vadjustment()
		start_y = vadj.get_value()
		return start_x,start_y

class NoteBook(Gtk.Notebook):
	def __init__(self):
		super(NoteBook,self).__init__()
	
	def add_page(self,title,widget):
		label = Gtk.Label(title)
		self.append_page(widget,label)


class Frame(Gtk.Frame):
	def __init__(self,label_text):
		super(Frame,self).__init__()
		self.set_label(label_text)


class Paned(Gtk.Paned):
	HORIZONTAL = Gtk.Orientation.HORIZONTAL;
	VERTICAL = Gtk.Orientation.VERTICAL;
	def __init__(self,orientation):
		super(Paned,self).__init__()
		self.set_orientation(orientation)
		
class Box(Gtk.Box):
	HORIZONTAL = Gtk.Orientation.HORIZONTAL;
	VERTICAL = Gtk.Orientation.VERTICAL;	
	def __init__(self,orientation):
		super(Box,self).__init__()
		self.set_orientation(orientation)

	def connect_configure_event_handler(self,function):
		self.connect("configure-event",function)

class Toolbar(Gtk.Toolbar):
	HORIZONTAL = Gtk.Orientation.HORIZONTAL;
	VERTICAL = Gtk.Orientation.VERTICAL;
	SEPARATOR = 1;

	def __init__(self,orientation,specification):
		super(Toolbar,self).__init__()
		self.set_orientation(orientation)
		for item in specification:
			if item == Toolbar.SEPARATOR:
				toolbar_item = Gtk.SeparatorToolItem()
			else:
				toolbar_item = Gtk.ToolButton(item[0])
				toolbar_item.connect("clicked",item[1])
				if item[0] in icon.stock_icon_dict.keys():
					image = Gtk.Image()
					image.set_from_file(macros.icon_dir+icon.stock_icon_dict[item[0]]+".png")
					toolbar_item.set_icon_widget(image)
				else:
					label = Gtk.Label(item[0])
					if(orientation == Gtk.Orientation.VERTICAL):
						label.set_angle(90)
					toolbar_item.set_icon_widget(label)
				toolbar_item.set_tooltip_text(item[0])
			self.add(toolbar_item)
	
	def set_show_nth_item(self,n,value):
		item = self.get_nth_item(n)
		item.set_sensitive(value)
			
		
		



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import GObject

from lios.ui.gtk import loop, dialog, widget, drawing_area, tree_view, containers, window

from lios import image_logics, localization
_ = localization._


class ImageViewer(containers.Paned):
	ZOOM_FIT = 4
	__gsignals__ = {
        'list_updated' : (GObject.SIGNAL_RUN_LAST,
                     GObject.TYPE_NONE,
                     ())
        }


	def __init__(self):
		super(ImageViewer,self).__init__(containers.Paned.HORIZONTAL)
		self.set_border_width(5)

		#Drawing Area		
		self.drawingarea = drawing_area.DrawingArea()
		
		self.scrolled = containers.ScrollBox()
		self.scrolled.add_with_viewport(self.drawingarea);
		
		
		
		self.drawingarea.connect_button_press_event(self.__drawingarea_button_press_event)
		self.drawingarea.connect_button_release_event(self.__drawingarea_button_release_event)
		self.drawingarea.connect_motion_notify_event(self.__drawingarea_motion_notify_event)
		

		self.add(self.scrolled)
				
		#Drawing List Tree View
		self.treeview = tree_view.TreeView([(_("Selected"),bool,False),
		(_("X"),float,True),(_("Y"),float,True),
		(_("Width"),float,True),(_("Height"),float,True),
		(_("Letter"),str,True)],self.edited_callback)
		
		self.treeview.connect_cursor_change_function(self.treeview_cursor_changed)
		self.treeview.connect_rows_reordered_function(self.treeview_rows_reordered)
		self.treeview.set_column_visible(0,False);
		self.treeview.set_reorderable(True)
		scrolled_treeview = containers.ScrollBox()
		scrolled_treeview.add(self.treeview)

		self.rs = []
		self.start_row_index = -1
		self.previus_row_index = -1


		button1 = widget.Button(_("_Delete"))
		button1.set_use_underline(True)
		button1.set_tooltip_text(_("Shortcut Alt+D"))
		button1.connect_function(self.__delete_selection)

		button2 = widget.Button(_("Clear"))
		button2.connect_function(self.clear_selection)

		grid = containers.Grid()
		grid.add_widgets([(scrolled_treeview,2,1,containers.Grid.HEXPAND,containers.Grid.VEXPAND),
		containers.Grid.NEW_ROW,(button1,1,1,containers.Grid.HEXPAND,containers.Grid.NO_VEXPAND),
		(button2,1,1,containers.Grid.HEXPAND,containers.Grid.NO_VEXPAND)])
		self.add(grid)
		

		
		#Inetial Values
		self.on_select = False
		self.on_resize = False
		self.zoom_list = [0.20,0.40,0.60,0.80,1,1.20,1.40,1.60,1.80,2]
		self.zoom_level = self.ZOOM_FIT
		self.drawingarea.show()
		
		self.set_position(400)
		self.show()
		
	
	#set_position()
	
	def scroll_image_view(self,h_value,v_value):
		self.scrolled.scroll(h_value,v_value)

	def get_image_view_size_on_screen(self):
		return self.scrolled.get_size_on_screen()

	def get_image_view_scrolled_start_points(self):
		return self.scrolled.get_current_start_points()

	def edited_callback(self,row):
		# Reset the new rectangle list
		rs = self.treeview.get_list()
		if(not image_logics.is_overlapping(
		[ [row[1],row[2],row[3],row[4] ] for row in self.rs ],
		row,rs[row][1],rs[row][2],rs[row][3],rs[row][4])):
			self.rs = rs;
			
			#applay new rectangle list on drawing area
			self.drawingarea.set_rectangle_list([[ row[0], row[1],row[2],row[3],row[4] ] for row in self.rs ])
			self.drawingarea.redraw()
			self.emit('list_updated')
		else:
			self.treeview.set_list(self.rs)
	
	def treeview_cursor_changed(self):
		selected_row = self.treeview.get_selected_row_index()
		self.set_selected_item(selected_row)				
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.drawingarea.redraw()

		#Scroll image view to new position
		width,height = self.get_image_view_size_on_screen()
		cur_rec_x = self.rs[selected_row][1]#+self.rs[selected_row][3]
		cur_rec_y = self.rs[selected_row][2]#+self.rs[selected_row][4]
		x = (cur_rec_x - (width*50/100));
		y = (cur_rec_y - (height*50/100));
		self.scroll_image_view(x,y);

		# Note : The set list function should not again - 
		# trigger cursor-change function 
		#self.treeview.set_list(self.rs)

	def treeview_rows_reordered(self):
		self.rs = self.treeview.get_list()
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.emit('list_updated')

	def load_image(self,filename,list,zoom_level):
		if(0 <= zoom_level <= 9):
			self.start_type = 0;
			self.filename = filename
			self.old_zoom_level = self.zoom_level
			diff = self.zoom_level - zoom_level
			self.zoom_level = zoom_level
			parameter = self.zoom_list[self.zoom_level]
			self.set_list(list,diff)
			self.drawingarea.load_image(filename,list,parameter);


	def set_list(self,list_,diff):
		if (list_ == None):
			old_factor = self.old_zoom_level - 4
			new_factor = self.zoom_level - 4

			list_ = []
			# Here we have to revert to the original because we always
			# zoom in/out on the original image with factors to keep the quality
			# ie zoom the image from original instead of zooming the zoomed image
			for item in self.rs:
				x_orig = item[1]*100/((old_factor*20)+100)
				x = x_orig+((x_orig*20*new_factor)/100)
				y_orig = item[2]*100/((old_factor*20)+100)
				y = y_orig+((y_orig*20*new_factor)/100)
				width_orig = item[3]*100/((old_factor*20)+100)
				width = width_orig+((width_orig*20*new_factor)/100)
				height_orig = item[4]*100/((old_factor*20)+100)
				height = height_orig+((height_orig*20*new_factor)/100)
				list_.append([0,x,y,width,height,item[5]])


		self.rs = list(list(x) for x in list_)
		self.treeview.set_list(self.rs)
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.drawingarea.redraw()

	def get_list(self):
		old_factor = self.zoom_level - 4
		list_ = []
		# Here we have to return the original because we always
		# zoom in/out on the original image with factors to keep the quality
		# ie zoom the image from original instead of zooming the zoomed image
		for item in self.rs:
			x_orig = item[1]*100/((old_factor*20)+100)
			y_orig = item[2]*100/((old_factor*20)+100)
			width_orig = item[3]*100/((old_factor*20)+100)
			height_orig = item[4]*100/((old_factor*20)+100)
			list_.append([0,x_orig,y_orig,width_orig,height_orig,item[5]])
		return list(list(x) for x in list_)

	def get_height(self):
		return self.drawingarea.get_height();

	def get_original_height(self):
		return self.drawingarea.get_original_height()

	def get_filename(self):
		return self.filename
	
	def set_label_entry_visible(self,value):
		self.treeview.set_column_visible(5,False);		

	def redraw(self):
		loop.acquire_lock()
		self.load_image(self.filename,[],self.ZOOM_FIT)
		loop.release_lock()

	def get_selection_list(self):
		# return with the pattern x, y , width, height, letter
		return([[ row[1],row[2],row[3],row[4], row[5] ] for row in self.rs ])
	
	# inorder to maintain the qulity the sub-image should be created from
	# original image so the rectangle coordinates should be in original points
	def save_sub_image(self,filename,x,y,width,height):
		factor = self.zoom_level - 4
		x = x*100/((factor*20)+100)
		y = y*100/((factor*20)+100)
		width = width*100/((factor*20)+100)
		height = height*100/((factor*20)+100)
		self.drawingarea.save_image_rectangle(filename,x,y,width,height)	
	
	def get_pixbuf(self):
		return self.pixbuf		
		   
	def zoom_in(self,data=None):
		self.load_image(self.filename,None, self.zoom_level + 1)
		self.treeview.set_list(self.rs)
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.emit('list_updated')

	def zoom_out(self,data=None):
		self.load_image(self.filename,None, self.zoom_level - 1)
		self.treeview.set_list(self.rs)
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.emit('list_updated')

	def zoom_fit(self,data=None):
		self.load_image(self.filename,None,self.ZOOM_FIT)
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.emit('list_updated')
	
	def get_zoom_level(self):
		return self.zoom_level




	def __drawingarea_button_press_event(self, point, button_type):
		if(button_type == 1):
			self.start_x,self.start_y=point
			width,height = self.get_image_view_size_on_screen()
			area_x_start, area_y_start = self.get_image_view_scrolled_start_points()
			self.start_type, self.start_row_index, self.start_position_type = image_logics.get_point_type(self.start_x,self.start_y, area_x_start, area_y_start, width, height,[ [row[1],row[2],row[3],row[4],row[0] ] for row in self.rs ])
		return True
    
	def __drawingarea_motion_notify_event(self, point):
		x,y = point;
		max_width = self.drawingarea.get_width()
		max_height = self.drawingarea.get_height()
		
		# 1 - Select
		if (self.start_type == 1):
			if(image_logics.detect_out_of_range(x,y,max_width,max_height)):
				return
			
			start_x,start_y,end_x,end_y = image_logics.order_rectangle(self.start_x,self.start_y,x,y);
			if(image_logics.detect_overlap([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],start_x,start_y,end_x,end_y)):
				return

			self.tmp_finish_x,self.tmp_finish_y = point
			self.drawingarea.set_drawing_rectangle((self.start_x,self.start_y,
			self.tmp_finish_x-self.start_x,self.tmp_finish_y-self.start_y))
			
			self.drawingarea.redraw()

		# 2 - Resize
		if (self.start_type == 2):
			if(image_logics.detect_out_of_range(x,y,max_width,max_height)):
				return
			
			if(self.start_position_type == 1):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				x,y,self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x),
				self.rs[self.start_row_index][4]+(self.rs[self.start_row_index][2] - y))):
					self.rs[self.start_row_index][4] = self.rs[self.start_row_index][4]+(self.rs[self.start_row_index][2] - y)
					self.rs[self.start_row_index][2] = y
					self.rs[self.start_row_index][3] = self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x)
					self.rs[self.start_row_index][1] = x
				
			elif(self.start_position_type == 2):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				self.rs[self.start_row_index][1],y,self.rs[self.start_row_index][3],y+self.rs[self.start_row_index][4] -y)):
					self.rs[self.start_row_index][4] = self.rs[self.start_row_index][4]+(self.rs[self.start_row_index][2] - y)
					self.rs[self.start_row_index][2] = y					

			elif(self.start_position_type == 3):
				if(not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				self.rs[self.start_row_index][1],y,
				x - self.rs[self.start_row_index][1],
				self.rs[self.start_row_index][4]+(self.rs[self.start_row_index][2] - y))):
					self.rs[self.start_row_index][4] = self.rs[self.start_row_index][4]+(self.rs[self.start_row_index][2] - y)
					self.rs[self.start_row_index][2] = y
					self.rs[self.start_row_index][3] = x - self.rs[self.start_row_index][1]		
															
			elif(self.start_position_type == 4):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				x,self.rs[self.start_row_index][2],
				self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x),
				self.rs[self.start_row_index][4])):
					self.rs[self.start_row_index][3] = self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x)
					self.rs[self.start_row_index][1] = x
				
			elif(self.start_position_type == 6):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				self.rs[self.start_row_index][1],self.rs[self.start_row_index][2],
				x - self.rs[self.start_row_index][1],self.rs[self.start_row_index][4])):
					self.rs[self.start_row_index][3] = x - self.rs[self.start_row_index][1]

			elif(self.start_position_type == 7):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				x,self.rs[self.start_row_index][2],
				self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x),
				y - self.rs[self.start_row_index][2])):
					self.rs[self.start_row_index][4] = y - self.rs[self.start_row_index][2]
					self.rs[self.start_row_index][3] = self.rs[self.start_row_index][3]+(self.rs[self.start_row_index][1] - x)
					self.rs[self.start_row_index][1] = x					

			elif(self.start_position_type == 8):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				self.rs[self.start_row_index][1],self.rs[self.start_row_index][2],
				self.rs[self.start_row_index][3],y - self.rs[self.start_row_index][2])):
					self.rs[self.start_row_index][4] = y - self.rs[self.start_row_index][2]		
		
			elif(self.start_position_type == 9):
				if( not image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,
				self.rs[self.start_row_index][1],self.rs[self.start_row_index][2],
				x - self.rs[self.start_row_index][1],y - self.rs[self.start_row_index][2])):
					self.rs[self.start_row_index][4] = y - self.rs[self.start_row_index][2]
					self.rs[self.start_row_index][3] = x - self.rs[self.start_row_index][1]
			
			self.treeview.set_list(self.rs)
			self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		

		# 3 - Moving
		if (self.start_type == 3):
			#Detect if anyof the new rectangle porsion is not feesible
			if(image_logics.detect_out_of_range(x - self.rs[self.start_row_index][3]/2,y - self.rs[self.start_row_index][4]/2,max_width,max_height) or
			image_logics.detect_out_of_range(x + self.rs[self.start_row_index][3]/2,y + self.rs[self.start_row_index][4]/2,max_width,max_height)):
				return
			
			if(image_logics.is_overlapping([ [row[1],row[2],row[3],row[4] ] for row in self.rs ],self.start_row_index,x - self.rs[self.start_row_index][3]/2,
			y - self.rs[self.start_row_index][4]/2,self.rs[self.start_row_index][3],self.rs[self.start_row_index][4])):
				return
			self.rs[self.start_row_index][1] = x - self.rs[self.start_row_index][3]/2;
			self.rs[self.start_row_index][2] = y - self.rs[self.start_row_index][4]/2;
			
			self.treeview.set_list(self.rs)
			self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		
		# 0 - Simply hovering 
		if (self.start_type == 0):
			area_x_start, area_y_start = self.get_image_view_scrolled_start_points()
			width,height = self.get_image_view_size_on_screen()
			_type, row_index, position_type = image_logics.get_point_type(x,y,area_x_start, area_y_start, width, height,[[row[1],row[2],row[3],row[4] ] for row in  self.rs ])
			self.drawingarea.set_mouse_pointer_type(position_type);

			# if self.previus_row_index not equll to row_index then the
			# mouse pointer moved from current box so we draw it

			if (self.previus_row_index != row_index):
				# While hovering over boxes the treeview cursor change handler function should not be called
				# because it simply scroll drawing area for each boxes which leads to flicker
				self.treeview.block_cursor_change_signal()
				if ( row_index != -1 ):
					# The mouse is over the box at row_index
					self.set_selected_item(row_index)
					self.treeview.set_list(self.rs);
					self.drawingarea.set_rectangle_list([[ row[0],row[0],row[1],row[2],row[3] ] for row in self.rs ])
					self.treeview.set_cursor(row_index)
				elif( self.start_row_index != -1 ):
					# If user made a selection then it should be preserved even after hovering other boxes
					self.set_selected_item(self.start_row_index)
					self.treeview.set_list(self.rs);
					self.drawingarea.set_rectangle_list([[ row[0],row[0],row[1],row[2],row[3] ] for row in self.rs ])
					self.treeview.set_cursor(self.start_row_index)

				self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
				self.treeview.unblock_cursor_change_signal()
			self.previus_row_index = row_index;
		self.drawingarea.redraw()

	def __drawingarea_button_release_event(self, point, button_type):
		if(self.start_type == 1):
			self.finish_x,self.finish_y=point
			self.drawingarea.set_drawing_rectangle(None)		
			
			#Swap coordinate if selected in reverse direction
			self.start_x,self.start_y,self.tmp_finish_x,self.tmp_finish_y = image_logics.order_rectangle(self.start_x,self.start_y,self.tmp_finish_x,self.tmp_finish_y)

			# finding the index of new box inside the box list
			index = image_logics.find_index_for_new_box(self.start_x,self.start_y,self.tmp_finish_x,self.tmp_finish_y,
			[[ row[1],row[2],row[3],row[4] ]  for row in self.rs ]);

			# Inserting the new box to the list
			self.rs.insert(index,[0,self.start_x,self.start_y,self.tmp_finish_x-self.start_x,self.tmp_finish_y-self.start_y,""])

			self.set_selected_item(len([ [row[1],row[2],row[3],row[4],row[0] ] for row in self.rs ])-1)
			self.treeview.set_list(self.rs)
			self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ]  for row in self.rs ]);
			self.drawingarea.redraw()

			self.emit('list_updated')
		
			

		# 2 - Resize or 3 - Moving
		if(self.start_type == 2 or self.start_type == 3):
			self.set_selected_item(self.start_row_index)
			self.treeview.set_list(self.rs);
			self.emit('list_updated')
			

		self.start_type = 0;
	
	def set_selected_item(self,row):
		self.selected_item = row;
		for i in range(0,len(self.rs)):
			if (i == row):
				self.rs[i][0] = 1;
			else:
				self.rs[i][0] = 0;	
	def get_selected_item_index(self):
		return self.selected_item;

	def __delete_selection(self,widget):
		list = []
		for item in self.rs:
			if (item[0] == 0):
				list.append(item)
		self.rs = list;
		# Set first element as selected 
		self.set_selected_item(0)
		# reset selection index
		self.start_row_index = -1;
		self.treeview.set_list(self.rs)
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ])
		self.drawingarea.redraw()
		self.emit('list_updated')

	def clear_selection(self,widget):
		self.rs = []
		self.treeview.clear()
		# reset selection index
		self.start_row_index = -1;
		self.drawingarea.set_rectangle_list([[ row[0],row[1],row[2],row[3],row[4] ] for row in self.rs ]);
		self.drawingarea.redraw()
		self.emit('list_updated')


	def connect_context_menu_button_callback(self,function):
		self.drawingarea.connect_context_menu_button_callback(function)


class TestWindow(window.Window):

    def __init__(self):
        window.Window.__init__(self, title="Hello World")
        
        self.iv = ImageViewer()
        li = [(0,10,10,10,10,"a"),(0,30,30,10,10,"b"),(0,50,50,10,10,"c")]
        self.iv.load_image("/usr/share/lios/lios.png",li,ImageViewer.ZOOM_FIT)

        button1 = widget.Button("Get List")
        button1.connect_function(self.on_button1_clicked)        

        button2 = widget.Button(label="Set List")
        button2.connect_function(self.on_button2_clicked)        

        button3 = widget.Button(label="Zoom-In")
        button3.connect_function(self.on_button3_clicked)        
        
        button4 = widget.Button(label="Zoom-Out")
        button4.connect_function(self.on_button4_clicked)
        
        grid = containers.Grid()
        grid.add_widgets([(self.iv,2,1,True,True),containers.Grid.NEW_ROW,
        (button1,1,1,True,False),(button2,1,1,True,False),containers.Grid.NEW_ROW,
        (button3,1,1,True,False),(button4,1,1,True,False)])        

        self.add(grid)
        grid.show()
        self.set_default_size(700,600);

    def on_button1_clicked(self, widget):
        print(self.iv.get_selection_list())

    def on_button2_clicked(self, widget):
        self.iv.set_selection_list([(25,10,45,60)])

    def on_button3_clicked(self, widget):
        self.iv.zoom_in()

    def on_button4_clicked(self, widget):
        self.iv.zoom_out()


if __name__ == "__main__":
	win = TestWindow()
	win.connect("delete-event", lambda x,y : loop.stop_main_loop())
	win.show_all()
	loop.start_main_loop()



#!/usr/bin/python3 
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import subprocess 
from lios.ocr.ocr_engine_base import OcrEngineBase

class OcrEngineAbbyyFineReader11(OcrEngineBase):
	name = "ABBYY FineReader11"
	
	def __init__(self,language=None):
		self.set_language(language)

	def is_available():
		if ("/bin/abbyyocr11" in subprocess.getoutput("whereis abbyyocr11")):
			return True
		else:
			return False
			
	def ocr_image_to_text(self,file_name):
		os.system("convert {} /tmp/{}_for_ocr.jpg".format(file_name,file_name.split("/")[-1]))
		os.system("abbyyocr11 -if /tmp/{0}_for_ocr.jpg -rl {1} -f TextUnicodeDefaults -tet UTF8 -of /tmp/{1}_output.txt ".format(file_name.split("/")[-1],self.language))
		print("abbyyocr11 -if /tmp/{0}_for_ocr.jpg -rl {1} -f TextUnicodeDefaults -tet UTF8 -of /tmp/{1}_output.txt ".format(file_name.split("/")[-1],self.language))
		os.remove("/tmp/{0}_for_ocr.jpg".format(file_name.split("/")[-1]))
		try:
			with open("/tmp/{0}_output.txt".format(file_name.split("/")[-1]),encoding="utf-8") as file:
				text = file.read().strip()
				os.remove("/tmp/{0}_output.txt".format(file_name.split("/")[-1]))
				return text
		except:
			return ""
	def cancel():
		os.system("pkill convert")
		os.system("pkill abbyyocr11")
		
	
	def get_available_languages():
		langs = ["Abkhaz","Adyghe","Afrikaans","Agul","Albanian","Altaic",
		"Arabic (Saudi Arabia)","Armenian (Eastern, Western, Grabar)","Avar",
		"Aymara","Azerbaijani (Cyrillic, Latin)","Bashkir","Basque","Belarusian",
		"Bemba","Blackfoot","Breton","Bugotu","Bulgarian","Buryat","Catalan",
		"Cebuano","Chamorro","Chechen","Chinese (Simplified, Traditional)",
		"Chukchee","Chuvash","Corsican","Crimean Tatar","Croatian","Crow",
		"Czech","Dakota","Danish","Dargwa","Dungan","Dutch (Netherlands and Belgium)",
		"English","Eskimo (Cyrillic, Latin)","Estonian","Even","Evenki","Faroese",
		"Fijian","Finnish","French","Frisian","Friulian","Gagauz","Galician","Ganda",
		"German (Luxemburg)","German (new and old spelling)","Greek","Guarani",
		"Hani","Hausa","Hawaiian","Hebrew","Hungarian","Icelandic","Indonesian",
		"Ingush","Irish","Italian","Japanese","Jingpo","Kabardian","Kalmyk",
		"Karachay-balkar","Karakalpak","Kasub","Kawa","Kazakh","Khakass",
		"Khanty","Kikuyu","Kirghiz","Kongo","Korean","Korean (Hangul)",
		"Koryak","Kpelle","Kumyk","Kurdish","Lak","Latin","Latvian","Lezgi",
		"Lithuanian","Luba","Macedonian","Malagasy","Malay","Malinke","Maltese",
		"Mansi","Maori","Mari","Maya","Miao","Minangkabau","Mohawk","Moldavian",
		"Mongol","Mordvin","Nahuatl","Nenets","Nivkh","Nogay",
		"Norwegian (Nynorsk and Bokmal)","Nyanja","Occitan","Ojibway","Ossetian",
		"Papiamento","Polish","Portuguese (Portugal and Brazil)","Provencal",
		"Quechua","Rhaeto-romanic","Romanian","Romany","Rundi","Russian",
		"Russian (old spelling)","Rwanda","Sami (Lappish)","Samoan","Scottish Gaelic",
		"Selkup","Serbian (Cyrillic, Latin)","Shona","Slovak","Slovenian",
		"Somali","Sorbian","Sotho","Spanish","Sunda","Swahili","Swazi","Swedish",
		"Tabasaran","Tagalog","Tahitian","Tajik","Tatar","Thai","Tok Pisin",
		"Tongan","Tswana","Tun","Turkish","Turkmen (Cyrillic, Latin)","Tuvinian",
		"Udmurt","Uighur (Cyrillic, Latin)","Ukrainian","Uzbek (Cyrillic, Latin)",
		"Vietnamese","Welsh","Wolof","Xhosa","Yakut","Yiddish","Zapotec","Zulu"]
		return langs



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################



from lios.ocr.ocr_engine_base import OcrEngineBase
from lios.ocr.ocr_engine_cuneiform import OcrEngineCuneiform
from lios.ocr.ocr_engine_tesseract import OcrEngineTesseract
from lios.ocr.ocr_engine_ocrad import OcrEngineOcrad
from lios.ocr.ocr_engine_gocr import OcrEngineGocr
from lios.ocr.ocr_engine_abbyy_finereader11 import OcrEngineAbbyyFineReader11
from lios.ocr.ocr_engine_abbyy_finereader9 import OcrEngineAbbyyFineReader9

def get_available_engines():
	list = []
	# Note : The engine classes should be sorted otherwise each time this function
	# will return same engine list in a random order.  
	for engine_name in ["Tesseract","Cuneiform","ABBYY FineReader9","ABBYY FineReader11",
						"Gocr", "Ocrad" ]:
		for item in OcrEngineBase.__subclasses__():
			if (item.name == engine_name and item.is_available()):
				list.append(item)
	
	import operator
	return list





#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2014 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import subprocess
from lios.scanner.driver_base import DriverBase

class DriverScanimage(DriverBase):
	name = "Scanimage"
	
	def __init__(self,device,scanner_mode_switching,resolution=300,brightness=40,scan_area=0):
		self.device = device.split()[1][1:-1]
		self.device_name = device;
		
		self.scanner_mode = "Color"
		self.light_parameter_state = False

		device_info = subprocess.getoutput("scanimage -d {} --all-options"\
		.format(self.device))		
		
		for line in device_info.split("\n"):
			if ("--mode" in line):
				 self.available_modes = line.split()[1].split("|")
				 print(self.available_modes)
			
			if ("--brightness" in line):
				self.light_parameter_state = True
				self.light_parameter = "brightness"
				print("Brightness available")
			
			#if ("--threshold" in line and self.light_parameter_state == False):
			#	self.light_parameter_state = True
			#	self.light_parameter = "threshold"
			#	print("Threshold available")

			if ("-y" in line):
				self.max_y = line.split()[1].split(".")[2].split("mm")[0]
				print(self.max_y)

			if ("-x" in line):
				self.max_x = line.split()[1].split(".")[2].split("mm")[0]
				print(self.max_x)

		super(DriverScanimage, self).__init__(device,scanner_mode_switching,
			resolution,brightness,scan_area)
		self.brightness_multiplier = 2
		self.brightness_offset = -100
				

	
	def scan(self,filename,brightness=-1,resolution=-1,scan_area=-1):
		super(DriverScanimage, self).scan(filename,brightness,resolution,scan_area)

		command = "scanimage --device-name='{}' --resolution {} --mode {} -x {} -y {}"\
		.format(self.device,self.resolution,self.scanner_mode,self.max_x,self.y)
		
		
			
		#if (self.calibration_cache):
		#	command += (" --calibration-cache=yes")
			
		if (self.check_brightness_support()):
			command += (" --{}={}".format(self.light_parameter,self.brightness))	
				
		command += (" > {}".format(filename))
		print(command)
		os.system(command)
		os.system("convert -strip -quality 90 {0} {0}".format(filename))
				


	def get_resolution(self):
		return self.resolution

	def set_resolution(self,resolution):
		self.resolution = resolution

	def get_brightness(self):
		return self.brightness

	def set_brightness(self,brightness):
		self.brightness = brightness

	def set_scan_area(self,scan_area):
		if scan_area == self.SCAN_AREA_FULL:
			self.y = int(self.max_y)
		elif scan_area == self.SCAN_AREA_THREE_QUARTER:
			self.y = 3*(int(self.max_y)/4)
		elif scan_area == self.SCAN_AREA_HALF:
			self.y = int(self.max_y)/2
		else:
			self.y = int(self.max_y)/4
				
	def get_scan_area(self):
		return self.scan_area

	def set_scan_mode(self,mode):
		self.scanner_mode = mode

	def get_scan_mode(self,scan_mode):
		return self.scanner_mode

	def get_available_scan_modes(self):
		return self.available_modes

	def check_brightness_support(self):
		return self.light_parameter_state

	#static method
	def get_available_devices():
		scanner_list = []
		output = subprocess.getoutput("scanimage --list")
		for line in output.split("\n"):
			if (("device" in line) and ("v4l" not in line)):
				try:
					scanner_list.append(line)
				except:
					pass
		return scanner_list

	#static method
	def is_available():
		if ("/bin/scanimage" in subprocess.getoutput("whereis scanimage")):
			return True
		else:
			return False
	
	def cancel(self):
		os.system("pkill scanimage")

	def close(self):
		return
		
		
if __name__ == "__main__":
	scanners = []
	scanner_list = DriverScanimage.get_available_devices()
	print(scanner_list)
	for device in scanner_list:
		scanners.append(DriverScanimage(device))
	scanners[0].scan("Hello.png")		

	



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
from lios import localization
_ = localization._

stock_icon_dict = { _("_File") :"edit-copy",
					_("New") :"document-new",
					_("Open"):"document-open",
					_("Save"):"document-save",
					_("Save-As"):"document-save-as",
					_("Find"):"find",
					_("Find-Replace"):"find-replace",
					_("Text-Cleaner"):"find-replace",
					_("Import"):"document-open",
					_("Export"):"document-save",
					_("Apply-From-Cursor"):"find-replace",
					_("Apply-Entire"):"find-replace",
					_("Train-Tesseract"):"system-upgrade",
					_("Spell-Check"):"check-spelling",
					_("Start-Reader"):"media-playback-start",
					_("Stop-Reader"):"media-playback-stop",
					_("Increase-Reader-Speed"):"go-up",
					_("Decrease-Reader-Speed"):"go-down",
					_("Stop-All-Process"):"media-playback-stop",
					_("Go-To-Page"):"go-jump",
					_("Go-To-Line"):"go-jump",
					_("Undo"):"edit-undo",
					_("Redo"):"edit-redo",
					_("Delete"):"remove",
					_("Punch-Text"):"insert-text",
					_("Append-Text"):"insert-text",
					_("Zoom-In"):"zoom-in",
					_("Zoom-Fit"):"zoom-fit",
					_("Zoom-Out"):"zoom-out",
					_("Rotate-Left"):"rotate-left",
					_("Rotate-Right"):"rotate-right",
					_("Rotate-Twice"):"flip-vertical",
					_("_Image"):"image",
					_("Import-Pdf"):"application-pdf",
					_("Import-Folder"):"folder-pictures",
					_("Import-Image"):"image",
					_("Invert-List"):"sort-descending",
					_("Recognize"):"convert",
					_("_Recognize"):"convert",
					_("Recognize-Selected-Images"):"convert",
					_("Recognize-All-Images"):"convert",
					_("Recognize-Selected-with-rotation"):"convert",
					_("Recognize-All-with-rotation"):"convert",
					_("Recognize-Selected-Areas"):"convert",
					_("Recognize-Current-Image"):"convert",
					_("Recognize-Current-Image-With-Rotation"):"convert",
					_("Clear"):"clear",
					_("Print"):"print",
					_("Export-Text-As-Pdf"):"application-pdf",
					_("Export-As-Pdf"):"application-pdf",					
					_("Print-Preview"):"print-preview",
					_("Quit"):"quit",
					_("_Edit"):"edit-cut",
					_("Cut"):"edit-cut",
					_("Copy"):"edit-copy",
					_("Paste"):"edit-paste",
					_("Bookmark"):"emblem-favorite",
					_("Bookmark-Table"):"emblem-favorite",
					_("Import-Bookmarks"):"emblem-favorite",
					_("Bookmark-Table-Complete"):"emblem-favorite",
					_("Update-Scanner-List"):"scanner",
					_("_Scan"):"scanner",
					_("Scan-Image"):"scanner",
					_("Scan-Image-Repeatedly"):"scanner",
					_("Scan-and-Ocr"):"scanner",
					_("Scan-and-Ocr-Repeatedly"):"scanner",
					_("Optimise-Scanner-Brightness"):"scanner",
					_("Scan-Using-Webcam"):"camera-web",
					_("Take-Screenshot"):"camera",
					_("Take-and-Recognize-Screenshot"):"camera",
					_("_Preferences"):"preferences",
					_("Preferences"):"preferences",
					_("Preferences-Scanning"):"preferences",
					_("Preferences-General"):"preferences",
					_("Preferences-Recognition"):"preferences",
					_("Restore"):"clear",
					_("Load"):"document-open",
					_("_Tools"):"accessories-text-editor",
					_("Dictionary"):"accessories-dictionary",
					_("Audio-Converter"):"audio-input-microphone",
					_("Help"):"info",
					_("Open-Readme"):"info",
					_("Video-Tutorials"):"info",
					_("Open-Home-Page"):"info",
					_("Get-Source-Code"):"info",
					_("About"):"about"}
					



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2014 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

try:
	import sane
	available = True
except:
	available = False
	
	
from lios.scanner.driver_base import DriverBase
import os

class DriverSane(DriverBase):
	name = "Sane"
	
	def __init__(self,device,scanner_mode_switching,resolution=300,brightness=40,scan_area=0):
		sane_version = sane.init()
		self.device_name = device[2];
		try:
			self.scanner = sane.open(device[0])
		except:
			print("####sane error!")
		else:
			#Brightness and Threshold
			self.light_parameter_state = False 
			options = self.get_scanner_option ('brightness')
			if options:
				self.light_parameter_state = True
				self.light_parameter = "brightness"
				try:
					self.min =  options[-1][0]
					self.max = options[-1][1]
				except:
					self.min = -100
					self.max = 100

			options = self.get_scanner_option ('threshold')
			if options:
				self.light_parameter_state = True
				self.light_parameter = "threshold"
				try:
					self.min =  options[-1][0]
					self.max = options[-1][1]
				except:
					self.min = 0
					self.max = 255
			
			#Calling super constructor for inetialising brightness resolution and scan area
			super(DriverSane, self).__init__(scanner_mode_switching,device,resolution,\
			brightness,scan_area)
			if(not scanner_mode_switching):
				self.set_scan_mode("Color")
			self.brightness_multiplier = (self.max - self.min)/100
			self.brightness_offset = self.min
			
	
	def get_scanner_option (self,name):
		options = self.scanner.get_options()
		for option in options:
			if option[1] == name:
				return option
		return False
		

	def scan(self,file_name,brightness=-1,resolution=-1,region=-1):
		super(DriverSane, self).scan(file_name,brightness,resolution,region)
		pil_image = self.scanner.scan()
		pil_image.save("/tmp/sane_temp.png")
		os.system("convert /tmp/sane_temp.png {}".format(file_name))

			
	def check_brightness_support(self):
		return self.light_parameter_state
	

	def get_resolution(self):
		return self.scanner.resolution;
		
	def set_resolution(self,resolution):
		self.scanner.resolution = resolution


	def get_brightness(self):
		if (self.check_brightness_support()):
			if self.light_parameter == "brightness":
				return self.scanner.brightness
			if self.light_parameter == "threshold":
				return self.scanner.threshold			
		else:
			return -1;

	def set_brightness(self,brightness):
		if (self.check_brightness_support()):
			print("Scanner Max = {0},  Scanner Min = {1}, Corected Value {2}".
				format(self.max,self.min,brightness))	
			
			if self.light_parameter == "brightness":
				try:
					self.scanner.brightness = brightness
				except AttributeError:
					print ("ooh")
			if self.light_parameter == "threshold":
				try:
					self.scanner.threshold = brightness
				except AttributeError:
					print ("Ohhh")	

	def set_scan_area(self,scan_area):
		#X Axis for scan Area
		option = self.get_scanner_option('br-x')
		if option:
			self.scanner.br_x = option[8][1]
		
		#Y Axis for scan Area
		option = self.get_scanner_option('br-y')
		if option:
			if scan_area == self.SCAN_AREA_FULL:
				self.scanner.br_y = option[8][1]
			elif scan_area == self.SCAN_AREA_THREE_QUARTER:
				self.scanner.br_y = 3*(option[8][1]/4)
			elif scan_area == self.SCAN_AREA_HALF:
				self.scanner.br_y = option[8][1]/2
			else:
				self.scanner.br_y = option[8][1]/4
			return True
		else:
			return False

	def get_scan_area(self):
		return self.scanner.br_y

		
	def set_scan_mode(self,scan_mode):
		self.scanner.mode = scan_mode


	def get_scan_mode(self,scan_mode):
		return self.scanner.mode


	def get_available_scan_modes(self):
		option = self.get_scanner_option("mode")
		if option:
			return option[-1:][0]
		else:
			return []


	#static method
	def get_available_devices():
		sane.init()
		list = []
		for device in sane.get_devices():
			if "scanner" in device[3]:
				list.append(device)
		return list			
	
	def is_available():
		return available
	
	def cancel(self):
		self.scanner.cancel()


	def close(self):
		self.scanner.close()

if __name__ == "__main__":
	scanners = []
	scanner_list = DriverSane.get_available_devices()
	print(scanner_list)
	for device in scanner_list:
		scanners.append(DriverSane(device))
	scanners[0].scan("Hello.png")

	



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk
from gi.repository import GdkPixbuf


class AboutDialog(Gtk.AboutDialog):

	def __init__(self,title,buttons):
		super(AboutDialog,self).__init__(title,None,True,buttons)
	
	def set_logo_from_file(self,filename):
		pixbuff =  GdkPixbuf.Pixbuf.new_from_file(filename)
		self.set_logo(pixbuff)
			
		
	#show_all()
	#run()



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2014 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
import abc

class DriverBase(metaclass=abc.ABCMeta):
	SCAN_AREA_FULL, SCAN_AREA_THREE_QUARTER, SCAN_AREA_HALF,\
	SCAN_AREA_QUARTER = range(4)
	
	@abc.abstractmethod
	def __init__(self,device,scanner_mode_switching,resolution=300,brightness=40,scan_area=0):
		self.set_resolution(resolution);
		self.set_brightness(brightness);
		self.set_scan_area(scan_area);
		self.brightness_multiplier = 1
		self.brightness_offset = 0
		if (scanner_mode_switching):
			modes_list = self.get_available_scan_modes()
			if "Binary" in modes_list:
				self.set_scan_mode("Binary")
			if "Lineart" in modes_list:
				self.set_scan_mode("Lineart")
	
	@abc.abstractmethod
	def scan(self,filename,resolution=-1,brightness=-1,scan_area=-1):
		if (brightness !=-1 ):
			brightness = (brightness*self.brightness_multiplier)+self.brightness_offset
			self.set_brightness(brightness)
		if (resolution !=-1 ):
			self.set_resolution(resolution);
		if (scan_area !=-1 ):
			self.set_scan_area(scan_area);			


	@abc.abstractmethod
	def get_resolution(self):
		return

	@abc.abstractmethod
	def set_resolution(self,resolution):
		return

	@abc.abstractmethod
	def get_brightness(self):
		return

	@abc.abstractmethod
	def set_brightness(self,brightness):
		return

	@abc.abstractmethod
	def set_scan_area(self,scan_area):
		return
		
	@abc.abstractmethod
	def get_scan_area(self):
		return

	@abc.abstractmethod
	def set_scan_mode(self,scan_mode):
		return

	@abc.abstractmethod
	def get_scan_mode(self,scan_mode):
		return

	@abc.abstractmethod
	def get_available_scan_modes(self):
		return;

	@abc.abstractmethod		
	def check_brightness_support(self):
		return

	@staticmethod
	@abc.abstractmethod
	def get_available_devices():
		return

	@staticmethod
	@abc.abstractmethod
	def is_available():
		return


	@abc.abstractmethod
	def cancel(self):
		return

	@abc.abstractmethod
	def close(self):
		return



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
from gi.repository import Gtk

class CellRendererSpin(Gtk.CellRendererSpin):
	def __init__(self,pos):
		super(CellRendererSpin,self).__init__()
		self.pos = pos

class CellRendererText(Gtk.CellRendererText):
	def __init__(self,pos):
		super(CellRendererText,self).__init__()
		self.pos = pos	

class CellRendererToggle(Gtk.CellRendererToggle):
	def __init__(self,pos):
		super(CellRendererToggle,self).__init__()
		self.pos = pos	
		
class TreeView(Gtk.TreeView):
	def __init__(self,name_type_tuple_list,function):
		super(TreeView,self).__init__()
		self.function = function;
		self.cursor_change_handler_id = None
		
		type_list = []
		i = 0;
		for item in name_type_tuple_list:
			type_list.append(item[1])
			if(item[1] == float):
				cell = CellRendererSpin(i)
				adjustment = Gtk.Adjustment(0, 0, 10000, 1, 10, 0)
				cell.set_property("adjustment", adjustment)
				cell.set_property("digits", 3)
				cell.connect("edited", self.on_float_edited)
			elif(item[1] == int):
				cell = CellRendererSpin(i)
				adjustment = Gtk.Adjustment(0, 0, 10000, 1, 10, 0)
				cell.set_property("adjustment", adjustment)
				cell.connect("edited", self.on_float_edited)
			elif(item[1] == bool):
				cell = CellRendererToggle(i)
				cell.set_radio(True)
				cell.connect("toggled", self.on_bool_edited)
			else:
				cell = CellRendererText(i)
				cell.connect("edited", self.on_edited)
			
			if(item[2] == True):
				cell.set_property("editable", True)
				
			col = Gtk.TreeViewColumn(item[0], cell, text=i)
			self.append_column(col)
			i = i + 1;
			
		self.rs = Gtk.ListStore(*type_list)
		self.set_model(self.rs);	
		
		self.set_reorderable(True)
		#self.connect("cursor-changed",self.__treeview_image_cursor_changed)
	
	def append(self,item):
		self.rs.append(item);

	def remove(self,index):
		iter = self.rs.get_iter(index)
		self.rs.remove(iter);
	
	def set_list(self,list):
		if(self.cursor_change_handler_id):
			self.handler_block(self.cursor_change_handler_id)
		self.rs.clear()
		for item in list:
			self.rs.append(item)
		if(self.cursor_change_handler_id):
			self.handler_unblock(self.cursor_change_handler_id)

	def block_cursor_change_signal(self):
		if(self.cursor_change_handler_id):
			self.handler_block(self.cursor_change_handler_id)

	def unblock_cursor_change_signal(self):
		if(self.cursor_change_handler_id):
			self.handler_unblock(self.cursor_change_handler_id)

	#def set_selected_row(self,row):
	#	self.handler_block(self.cursor_change_handler_id)
	#	self.set_cursor(row);
	#	self.handler_unblock(self.cursor_change_handler_id)
	
	def get_list(self):
		# list comprehension used to unpack Gtk.ListStore
		return [[ i for i in item ] for item in self.rs] 
		
	def clear(self):
		self.rs.clear()
	
	def on_float_edited(self, widget, path, value):
		self.rs[path][widget.pos] = float(value)
		self.function(int(path))

	def on_bool_edited(self, widget, path):
		self.rs[path][widget.pos] = True
		self.function(int(path))

	def on_edited(self, widget, path, value):
		self.rs[path][widget.pos] = str(value)
		self.function(int(path))
	
	def connect_update_callback(self,function):
		self.function = function
	
	def connect_cursor_change_function(self,function):
		self.cursor_change_handler_id = self.connect("cursor-changed", lambda x: function())

	def connect_rows_reordered_function(self,function):
		self.rows_reordered_handler_id = self.connect("drag-end", lambda x,y: function())
		
	def get_selected_row_index(self):
		item = self.get_selection()
		model,iter = item.get_selected()
		path = item.get_selected_rows()[0]
		if (iter == None):
			return 0;
		path = path.get_path(iter)
		return(path.get_indices()[0]);
	
	def set_column_visible(self,column_number, value):
		column = self.get_column(column_number);
		column.set_visible(value);
		
		

class TestWindow(Gtk.Window):
    def function(self,row):
		   print(row)
		   print(self.tv.get_list()) 
    def __init__(self):
        Gtk.Window.__init__(self, title="Hello World")
        
        self.tv = TreeView([("X",float,True),("Y",float,True),("Width",float,True),("Height",float,True),("Letter",str,True)],self.function)
        self.tv.append([10,20,40,40,"a"]);
        self.tv.append([20,30,40,40,"b"]);


        self.box = Gtk.VBox(spacing=6)
        self.add(self.box)
        self.box.pack_start(self.tv, True, True, 0)

        self.button1 = Gtk.Button(label="Get List")
        self.button1.connect("clicked", self.on_button1_clicked)        
        self.box.pack_start(self.button1, False, False, 0)

        self.button2 = Gtk.Button(label="Set List")
        self.button2.connect("clicked", self.on_button2_clicked)        
        self.box.pack_start(self.button2, False, False, 0)

        self.button3 = Gtk.Button(label="Letter Show")
        self.button3.connect("clicked", self.on_button3_clicked)        
        self.box.pack_start(self.button3, False, False, 0)

        self.button4 = Gtk.Button(label="Letter Hide")
        self.button4.connect("clicked", self.on_button4_clicked)        
        self.box.pack_start(self.button4, False, False, 0)

        self.add(self.box)
        self.set_default_size(400,400);
        
    def on_button1_clicked(self, widget):
		   print(self.tv.get_list())

    def on_button2_clicked(self, widget):
        self.tv.set_list([(25,10,45,60)])
    
    def on_button3_clicked(self, widget):
		   self.tv.set_column_visible(4,True);
		   
    def on_button4_clicked(self, widget):
		   self.tv.set_column_visible(4,False);
               



if __name__ == "__main__":
	win = TestWindow()
	win.connect("delete-event", Gtk.main_quit)
	win.show_all()
	Gtk.main()



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk, GObject, Vte
from gi.repository import GLib
from gi.repository import Gdk

class Terminal(Vte.Terminal):
	def __init__(self,path):
		super(Terminal,self).__init__()
		if hasattr(self, 'spawn_sync'):
			self.spawn_sync(Vte.PtyFlags.DEFAULT, #default is fine
			path, #where to start the command?
			["/bin/sh"], #where is the emulator?
			[], #it's ok to leave this list empty
			GLib.SpawnFlags.DO_NOT_REAP_CHILD,
			None, #at least None is required
			None);
		else:
			self.fork_command_full(Vte.PtyFlags.DEFAULT, #default is fine
			path, #where to start the command?
			["/bin/sh"], #where is the emulator?
			[], #it's ok to leave this list empty
			GLib.SpawnFlags.DO_NOT_REAP_CHILD,
			None, #at least None is required
			None);

	def run_command(self,command):
		command = command+"\n"
		length = len(command)
		self.feed_child(command, length)

	def connect_child_exit(self,function):
		self.connect ("child-exited", function)  

	def connect_context_menu_button_callback(self,function):
		def fun(widget,event):
			if ((event.type == Gdk.EventType.BUTTON_RELEASE and event.button == 3) or
				(event.type == Gdk.EventType.KEY_PRESS and event.hardware_keycode == 135)):
				function()
		self.connect("button-release-event",fun)
		self.connect("key-press-event",fun)

class TheWindow(Gtk.Window):

    def __init__(self):
        Gtk.Window.__init__(self, title="inherited cell renderer")
        self.set_default_size(600, 300)
        self.terminal     = Terminal("~/home")
        
        
        #Set up a button to click and run a demo command
        self.button = Gtk.Button("Do The Command")
        self.button.connect("clicked", self.InputToTerm)
        #end demo command code

        #set up the interface
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.pack_start(self.button, False, True, 0)
        #a scroll window is required for the terminal
        scroller = Gtk.ScrolledWindow()
        scroller.set_hexpand(True)
        scroller.set_vexpand(True)
        scroller.add(self.terminal)
        box.pack_start(scroller, False, True, 2)
        self.add(box)

    def InputToTerm(self, clicker):
	    command = "echo \"Sending this command to a virtual terminal.\""
	    self.terminal.run_command(command);


if (__name__ == "__main__"):
	win = TheWindow()
	win.connect("delete-event", Gtk.main_quit)
	win.show_all()
	Gtk.main()



#! /usr/bin/python3 
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import abc
import multiprocessing


class OcrEngineBase(metaclass=abc.ABCMeta):
	def __init__(self,language=None):
		self.language = language
	
	@staticmethod
	@abc.abstractmethod
	def get_available_languages():
		return
	
	@abc.abstractmethod
	def ocr_image_to_text(self,image_file_name):
		pass
	
	def cancel():
		pass
		

	def set_language(self,language):
		if language in self.__class__.get_available_languages():
			self.language = language
			return True
		else:
			return False
	
	def ocr_image_to_text_with_multiprocessing(self,image_file_name):
		parent_conn, child_conn = multiprocessing.Pipe()
		
		p = multiprocessing.Process(target=(lambda parent_conn, child_conn,
		image_file_name : child_conn.send(self.ocr_image_to_text(image_file_name))),
		args=(parent_conn, child_conn,image_file_name))
		
		p.start()
		p.join()
		
		return parent_conn.recv();


	@staticmethod
	@abc.abstractmethod
	def is_available():
		return		



#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
#!/usr/bin/env python3


from lios.scanner.driver_base import DriverBase
from lios.scanner.sane_driver import DriverSane
from lios.scanner.scanimage_driver import DriverScanimage

def get_available_drivers():
	list = []
	for item in DriverBase.__subclasses__():
		if item.is_available():
			list.append(item)
	return list




#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
import os


class IconView(Gtk.IconView):
	def __init__(self):
		super(IconView,self).__init__()
		self.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
		self.liststore_images = Gtk.ListStore(GdkPixbuf.Pixbuf, str)
		self.set_pixbuf_column(0)
		self.set_text_column(1)
		self.set_columns(1)
		self.set_model(self.liststore_images)

	#methord is not thread safe
	def add_item(self,filename):
		try:
			pixbuff =  GdkPixbuf.Pixbuf.new_from_file(filename)
		except:
			pass
		else:
			height = pixbuff.get_height()
			width = pixbuff.get_width()
			ratio = (height*50)/width
			buff = pixbuff.scale_simple(50,ratio,GdkPixbuf.InterpType.BILINEAR)
			del pixbuff
			self.liststore_images.append([buff, filename])
			self.queue_draw()
			del buff

	def remove_selected_items(self,remove_file_too=True):
		for item in self.get_selected_items():
			iter = self.liststore_images.get_iter_from_string(item.to_string())
			if(remove_file_too):
				os.remove(self.liststore_images.get_value(iter, 1))
			self.liststore_images.remove(iter)
	
	def select_all_items(self):
		self.select_all()

	def select_item(self,filename):
		model = self.get_model()
		#iter = model.get_iter_first()
		for item in self.get_selected_items():
			iter = self.liststore_images.get_iter_from_string(item.to_string())
			if (filename == self.liststore_images.get_value(iter, 1)):		
				path = model.get_path(iter)
				self.select_path(path)
				break;
				
	
	def reload_preview(self,filename):
		for item in self.liststore_images:
			if (item[1] == filename):
				pixbuff =  GdkPixbuf.Pixbuf.new_from_file(filename)
				height = pixbuff.get_height()
				width = pixbuff.get_width()
				ratio = (height*50)/width
				buff = pixbuff.scale_simple(50,ratio,GdkPixbuf.InterpType.BILINEAR)
				del pixbuff
				item[0] = buff
				del buff
		
	
	def get_selected_item_names(self):
		items = []
		for item in reversed(self.get_selected_items()):
			items.append(self.liststore_images[item[0]][1])
		return items;

	def invert_list(self,*data):
		liststore = Gtk.ListStore(GdkPixbuf.Pixbuf, str)
		for item in reversed(self.liststore_images):
			liststore.append((item[0],item[1]))
		self.liststore_images = liststore
		self.set_model(self.liststore_images)
	
	def connect_on_selected_callback(self,function):
		self.connect("selection-changed",function)
	
	def connect_context_menu_button_callback(self,function):
		def fun(widget,event):
			if ((event.type == Gdk.EventType.BUTTON_RELEASE and event.button == 3) or
				(event.type == Gdk.EventType.KEY_PRESS and event.hardware_keycode == 135)):
				function()
		self.connect("button-release-event",fun)
		self.connect("key-press-event",fun)




#!/usr/bin/python3 
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import os
import subprocess
from lios.ocr.ocr_engine_base import OcrEngineBase

TESSDATA_POSSIBLE_PATHS = [
	"/usr/share/tesseract-ocr/tessdata",
	"/usr/share/tesseract/tessdata",
	"/usr/share/tessdata",
	"/usr/local/share/tesseract-ocr/tessdata",
	"/usr/local/share/tesseract/tessdata",
	"/usr/local/share/tessdata",
	"/usr/share/tesseract-ocr/4.00/tessdata" ]

TESSDATA_EXTENSION = ".traineddata"

class OcrEngineTesseract(OcrEngineBase):
	name = "Tesseract"
	
	def __init__(self,language=None):
		self.set_language(language)

	def is_available():
		if ("/bin/tesseract" in subprocess.getoutput("whereis tesseract")):
			return True
		else:
			return False

	def is_training_executables_available():
		for executable_name in ["combine_tessdata","unicharset_extractor","shapeclustering","mftraining","cntraining","text2image"]:
			if not ("/bin/"+executable_name in subprocess.getoutput("whereis {0}".format(executable_name))):
				return False
		return True

	def ocr_image_to_text(self,file_name):
		os.system("convert {} -background white -flatten +matte /tmp/{}_for_ocr.png".format(file_name,file_name.split("/")[-1]))

		os.system("tesseract /tmp/{0}_for_ocr.png /tmp/{0}_output -l {1}".format(file_name.split("/")[-1],self.language))
		os.remove("/tmp/{0}_for_ocr.png".format(file_name.split("/")[-1]))
		
		try:
			with open("/tmp/{0}_output.txt".format(file_name.split("/")[-1]),encoding="utf-8") as file:
				text = file.read().strip()
				os.remove("/tmp/{0}_output.txt".format(file_name.split("/")[-1]))
				return text
		except:
			return ""
	def cancel():
		os.system("pkill convert")
		os.system("pkill tesseract")
		
	
	def get_available_languages_in_dirpath(dirpath):
		langs = []
		if os.access(dirpath, os.R_OK):
			for filename in os.listdir(dirpath):
				if filename.lower().endswith(TESSDATA_EXTENSION):
					lang = filename[:(-1 * len(TESSDATA_EXTENSION))]
					langs.append(lang)
		return langs

	def get_available_languages():
		langs = []
		for dirpath in TESSDATA_POSSIBLE_PATHS[::-1]:
			if (os.path.isfile(dirpath+"/configs/box.train")):
				for item in OcrEngineTesseract.get_available_languages_in_dirpath(dirpath):
					langs.append(item)
				return sorted(langs)
		return langs

	def get_all_available_dirs():
		result = []
		for root, dirs, files in os.walk("/"):
			if "tessdata" in dirs:
				dir = os.path.join(root, "tessdata")
				if (os.path.isfile(dir+"/configs/box.train")):
					result.append(dir)

		# Sorting according to possible list
		# [::-1] is used to reverse
		for path in TESSDATA_POSSIBLE_PATHS[::-1]:
			if (path in result):
				result.insert(0, result.pop(result.index(path)))
		return result

	def get_available_dirs():
		dir_list = [];
		for path in TESSDATA_POSSIBLE_PATHS:
			if (os.path.exists(path)):
				if (os.path.isfile(path+"/configs/box.train")):
					dir_list.append(path);
		return dir_list;



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk
from lios.ui.gtk import icon		
from lios import macros

SEPARATOR = 1;

class Menu(Gtk.Menu):
	def __init__(self):
		super(Menu,self).__init__()


class ImageMenuItem(Gtk.ImageMenuItem):
	def __init__(self,label):
		super(ImageMenuItem,self).__init__(label)

class SeparatorMenuItem(Gtk.SeparatorMenuItem):
	def __init__(self):
		super(SeparatorMenuItem,self).__init__()


class MenuBar(Gtk.MenuBar):
	def __init__(self,item_list):
		super(MenuBar,self).__init__()
		self.agr = Gtk.AccelGroup()
		for item in item_list:
			menu = create_menu(item,self.agr)
			self.append(menu)

	def get_accel_group(self):
		return self.agr
			
		
def create_menu(item,agr):
	if (type(item) == list ):
		menu_item = ImageMenuItem(item[0])
		menu = Gtk.Menu()
		if (item[0] in icon.stock_icon_dict.keys()):
			image = Gtk.Image()
			#image.set_from_icon_name(icon.stock_icon_dict[item[0]],10)
			image.set_from_file(macros.icon_dir+icon.stock_icon_dict[item[0]]+".png")
			menu_item.set_image(image)
			menu_item.set_always_show_image(True)
		
		for i in item[1:]:
			sub_menu = create_menu(i,agr)
			menu.append(sub_menu)
		menu_item.set_submenu(menu)
		menu_item.set_use_underline(True)
		return menu_item
	
	else:
		if(item == SEPARATOR):
			terminal_menu_item = SeparatorMenuItem()
			return (terminal_menu_item)
		else:
			terminal_menu_item = ImageMenuItem(item[0])
			if (item[0] in icon.stock_icon_dict.keys()):
				image = Gtk.Image()
				#image.set_from_icon_name(icon.stock_icon_dict[item[0]],10)
				image.set_from_file(macros.icon_dir+icon.stock_icon_dict[item[0]]+".png")
				terminal_menu_item.set_image(image)
				terminal_menu_item.set_always_show_image(True)
			terminal_menu_item.connect("activate",item[1])
				
			if(item[2] != "None"):
				key, mod = Gtk.accelerator_parse(item[2])
				terminal_menu_item.add_accelerator("activate", agr, key,
				mod, Gtk.AccelFlags.VISIBLE)
			return (terminal_menu_item)



#** Context Menu related classes and functions **#

class MenuItem(Gtk.MenuItem):
	def __init__(self,label):
		super(MenuItem,self).__init__(label)


class ContextMenu(Gtk.Menu):
	def __init__(self,item_list):
		super(ContextMenu,self).__init__()
		for item in item_list:
			menu = create_context_menu(item)
			self.append(menu)
	def pop_up(self,*data):
		self.popup(None,None,None,None,0,0)
		self.show_all()

def create_context_menu(menu_list):
	if (type(menu_list) == list ):
		menu_item = MenuItem(menu_list[0])
		menu = Gtk.Menu()
		for item in menu_list[1:]:
			sub_menu = create_context_menu(item)
			menu.append(sub_menu)
		menu_item.set_submenu(menu)
		return menu_item
	else:
		if(menu_list == SEPARATOR):
			terminal_menu_item = SeparatorMenuItem()
		else:
			terminal_menu_item = MenuItem(menu_list[0])
			terminal_menu_item.connect("activate",menu_list[1])				
		return (terminal_menu_item)




#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk		
from gi.repository import Gdk		
from gi.repository import GdkPixbuf
		



class DrawingArea(Gtk.DrawingArea):
	def __init__(self):
		super(DrawingArea,self).__init__()
		self.set_events(Gdk.EventMask.ALL_EVENTS_MASK)
		self.connect("draw",self.__drawingarea_draw)
		self.rectangles = [];
		self.drawing_rectangle = None;
	
	def set_rectangle_list(self,list):
		self.rectangles = list;
	
	def set_drawing_rectangle(self,rectangle):
		self.drawing_rectangle = rectangle
	
	def connect_button_press_event(self,function):
		self.connect("button_press_event",lambda x,y: function(y.get_coords(),y.button))

	def connect_button_release_event(self,function):
		self.connect("button_release_event",lambda x,y: function(y.get_coords(),y.button))

	def connect_motion_notify_event(self,function):
		self.connect("motion_notify_event",lambda x,y: function(y.get_coords()))

	def __drawingarea_draw(self, widget, cr):
		   Gdk.cairo_set_source_pixbuf(cr, self.pixbuf, 0, 0)
		   cr.paint()
		   
		   for item in self.rectangles:
			   #cr.move_to(10, 90)
			   cr.rectangle(item[1], item[2], item[3], item[4])

			   # Red color for selected item blue for unselected items
			   if item[0] == True:
				   cr.set_source_rgb(0.9, 0, 0)
			   else:
				   cr.set_source_rgb(0, 0.1, 1)

			   cr.set_line_width (2.0);
			   #cr.fill()
			   cr.stroke()
		   
		   if (self.drawing_rectangle):
			   cr.rectangle(self.drawing_rectangle[0],self.drawing_rectangle[1],self.drawing_rectangle[2],self.drawing_rectangle[3])
			   # Green Color for currently drawing rectangle
			   cr.set_source_rgb(0, 1.0, 0)
			   cr.set_line_width (2.0);
			   cr.stroke()
		   return True
	def save_image_rectangle(self,filename,x,y,width,height):
		new_pixbuf = self.pixbuf_original.new_subpixbuf(x,y,width,height)
		new_pixbuf.savev(filename, "png",[],[])		

	
	def get_width(self):
		return self.pixbuf.get_width()		

	def get_height(self):
		return self.pixbuf.get_height()
	
	def redraw(self):
		self.queue_draw();		

	def load_image(self,filename,list,parameter):
		self.filename = filename
		
		self.pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
		self.pixbuf_file_name = filename

		# Keeping a original copy, later used to create sub-images
		self.pixbuf_original = self.pixbuf.copy();

		width = self.pixbuf.get_width()
		height = self.pixbuf.get_height()
		self.orig_height = height

		if (parameter != 1 ):
			self.set_size_request(width*parameter,height*parameter)
			self.pixbuf = self.pixbuf.scale_simple(width*parameter,height*parameter,GdkPixbuf.InterpType.HYPER)
		else:
			self.set_size_request(width,height)
		
		self.rectangles = list;	
		self.queue_draw()
	
	def get_original_height(self):
		return self.orig_height

	def set_mouse_pointer_type(self,_type):
		list = [Gdk.CursorType.ARROW,
		Gdk.CursorType.TOP_LEFT_CORNER,Gdk.CursorType.SB_V_DOUBLE_ARROW,Gdk.CursorType.TOP_RIGHT_CORNER,
		Gdk.CursorType.SB_H_DOUBLE_ARROW,Gdk.CursorType.FLEUR,Gdk.CursorType.SB_H_DOUBLE_ARROW,
		Gdk.CursorType.BOTTOM_LEFT_CORNER,Gdk.CursorType.SB_V_DOUBLE_ARROW,Gdk.CursorType.BOTTOM_RIGHT_CORNER]	
		arrow = Gdk.Cursor(list[_type])
		gdk_window = self.get_root_window()
		gdk_window.set_cursor(arrow)

	def connect_context_menu_button_callback(self,function):
		def fun(widget,event):
			if ((event.type == Gdk.EventType.BUTTON_RELEASE and event.button == 3) or
				(event.type == Gdk.EventType.KEY_PRESS and event.hardware_keycode == 135)):
				function()
		self.connect("button-release-event",fun)
		self.connect("key-press-event",fun)





#!/usr/bin/env python3
###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2011-2015 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################


import configparser
import shutil
from lios.ui.gtk import widget
from lios.ui.gtk import containers
from lios.ui.gtk import loop
from lios.ui.gtk import dialog

from lios import speech
from lios import localization

_ = localization._


class lios_preferences:
	def __init__(self):
		
		#Setting Default Values
		self.font="Georgia 14";self.highlight_font="Georgia 14";
		self.background_color="#000";self.font_color="#fff";
		self.highlight_color="#1572ffff0000";
		self.background_highlight_color="#00000bacffff";
		self.speech_module=0;self.speech_language=10;
		self.speech_rate=0;self.speech_pitch=0;self.speech_volume=100;
		self.time_between_repeated_scanning=0;self.scan_resolution=300;
		self.scan_driver=1;self.scanner_cache_calibration=0;
		self.scan_brightness=50;self.scan_area=0;self.insert_position=2;
		self.ocr_engine=0;self.language=0;self.mode_of_rotation=0;
		self.number_of_pages_to_scan=100;self.give_page_number = 1;
		self.page_numbering_type=0;self.starting_page_number=1;
		self.scanner_mode_switching=0;
		self.run_text_cleaner=0;self.rotation_angle=00;


	def set_avalable_scanner_drivers(self,list):
		self.available_scanner_drivers = list

	def set_avalable_ocr_engines(self,list):
		self.available_ocr_engine_list = list
		
	# FUNCTION TO Read PREFERENCES #
	def set_from_file(self,filename):
		config = configparser.ConfigParser()
		if config.read(filename) != []:
			try:
				self.time_between_repeated_scanning=int(config.get('cfg',"time_between_repeated_scanning"))
				self.scan_resolution=int(config.get('cfg',"scan_resolution"))
				self.scan_brightness=int(config.get('cfg',"scan_brightness"))
				self.ocr_engine=int(config.get('cfg',"ocr_engine"))
				self.scan_area=int(config.get('cfg',"scan_area"))
				self.scan_driver=int(config.get('cfg',"scan_driver"))
				self.insert_position=int(config.get('cfg',"insert_position"))
				self.language=int(config.get('cfg',"language"))
				self.speech_module=int(config.get('cfg',"speech_module"))
				self.speech_language=int(config.get('cfg',"speech_language"))
				self.speech_rate=int(config.get('cfg',"speech_rate"))
				self.speech_volume=int(config.get('cfg',"speech_volume"))
				self.speech_pitch=int(config.get('cfg',"speech_pitch"))
				self.number_of_pages_to_scan=int(config.get('cfg',"number_of_pages_to_scan"))#pages
				self.mode_of_rotation = int(config.get('cfg',"mode_of_rotation"))
				self.rotation_angle = int(config.get('cfg',"rotation_angle"))
				self.give_page_number=int(config.get('cfg',"give_page_number"))
				self.page_numbering_type=int(config.get('cfg',"numbering_type"))
				self.scanner_mode_switching=int(config.get('cfg',"scanner_mode_switching"))
				self.run_text_cleaner=int(config.get('cfg',"run_text_cleaner"))
				self.scanner_cache_calibration=int(config.get('cfg',"scanner_cache_calibration"))						
				self.starting_page_number=int(config.get('cfg',"starting_page_number"))
				self.background_color=config.get('cfg',"background_color")
				self.font_color=config.get('cfg',"font_color")
				self.highlight_color=config.get('cfg',"highlight_color")
				self.background_highlight_color=config.get('cfg',"highlight_background_color")
				self.font=config.get('cfg',"font")
				self.highlight_font=config.get('cfg',"highlight_font")				
				self.require_scanner_refresh = True
			except:
				self.__init__()
		else:
			self.__init__()

	def save_to_file(self,filename):
		#Removing old configuration file
		try:
			os.remove(filename)
		except:
			pass		
		config = configparser.ConfigParser()
		config.add_section('cfg')
		config.set('cfg',"time_between_repeated_scanning",str(self.time_between_repeated_scanning))
		config.set('cfg',"scan_resolution",str(self.scan_resolution))
		config.set('cfg',"scan_brightness",str(self.scan_brightness))
		config.set('cfg',"ocr_engine",str(self.ocr_engine))
		config.set('cfg',"insert_position",str(self.insert_position))
		config.set('cfg',"scan_area",str(self.scan_area))
		config.set('cfg',"scan_driver",str(self.scan_driver))
		config.set('cfg',"language",str(self.language))
		config.set('cfg',"speech_module",str(self.speech_module))
		config.set('cfg',"speech_language",str(self.speech_language))
		config.set('cfg',"speech_pitch",str(self.speech_pitch))
		config.set('cfg',"speech_volume",str(self.speech_volume))
		config.set('cfg',"speech_rate",str(self.speech_rate))
		config.set('cfg',"number_of_pages_to_scan",str(self.number_of_pages_to_scan))
		config.set('cfg',"mode_of_rotation",str(self.mode_of_rotation))
		config.set('cfg',"rotation_angle",str(self.rotation_angle))
		config.set('cfg',"give_page_number",str(self.give_page_number))
		config.set('cfg',"numbering_type",str(self.page_numbering_type))
		config.set('cfg',"scanner_mode_switching",str(self.scanner_mode_switching))
		config.set('cfg',"run_text_cleaner",str(self.run_text_cleaner))
		config.set('cfg',"scanner_cache_calibration",str(self.scanner_cache_calibration))				
		config.set('cfg',"starting_page_number",str(self.starting_page_number))
		config.set('cfg',"background_color",str(self.background_color))
		config.set('cfg',"font_color",str(self.font_color))
		config.set('cfg',"highlight_color",str(self.highlight_color))
		config.set('cfg',"highlight_background_color",str(self.background_highlight_color))
		config.set('cfg',"font",str(self.font))
		config.set('cfg',"highlight_font",str(self.highlight_font))
		with open(filename, 'w') as configfile:
			config.write(configfile)


	def update_page_number(self):
		loop.acquire_lock()
		if (self.page_numbering_type == 0):
			self.starting_page_number = self.starting_page_number + 1
		else:
			self.starting_page_number = self.starting_page_number + 2
		loop.release_lock()

	def get_page_number_as_string(self):
		if (self.page_numbering_type == 0):
			return ("{0}".format(self.starting_page_number))
		else:
			return ("{0}-{1}".format(self.starting_page_number,self.starting_page_number + 1))
	
	
	#Function for manipulating preferences		
	def open_configure_dialog(self,page=0):
		def change_engine(*data):
			index_engine = combobox_engine.get_active()
			combobox_language.clear()
			for item in self.available_ocr_engine_list[index_engine][1]:
				combobox_language.add_item(item)
			combobox_language.set_active(self.language)

		def change_speech_module(*data):
			index_engine = combobox_speech_module.get_active()
			combobox_speech_language.clear()
			test = speech.Speech()
			list = test.list_output_modules()
			test.set_output_module(list[index_engine])
			for item in test.list_voices():
				combobox_speech_language.add_item(item)
			combobox_speech_language.set_active(self.speech_language)

		def change_mode_of_rotation(*data):
			if(combobox_mode_of_rotation.get_active() == 2):
				combobox_angle.show()
				self.label_angle.show()
			else:
				combobox_angle.hide()
				self.label_angle.hide()


		self.require_scanner_refresh = False
		
		#Notebook
		notebook = containers.NoteBook()
		notebook.show_all()
		
		#GENERAL - PAGE #########	
		label_font = widget.Label(_("Font"))
		fontbutton_font = widget.FontButton()
		fontbutton_font.set_font_name(self.font)
		label_font.set_mnemonic_widget(fontbutton_font)

		label_font_color = widget.Label(_("Font Color"))
		colorbutton_font = widget.ColorButton()
		colorbutton_font.set_color_from_string(self.font_color)
		label_font_color.set_mnemonic_widget(colorbutton_font)
		

		label_background_color = widget.Label(_("Background Color"))
		colorbutton_background = widget.ColorButton()
		colorbutton_background.set_color_from_string(self.background_color)
		label_background_color.set_mnemonic_widget(colorbutton_background)
		

		label_highlight_font = widget.Label(_("Highlight Font"))
		fontbutton_highlight_font = widget.FontButton()
		fontbutton_highlight_font.set_font_name(self.highlight_font)
		label_highlight_font.set_mnemonic_widget(fontbutton_highlight_font)
		
		label_highlight_color = widget.Label(_("Highlight Color"))
		colorbutton_highlight = widget.ColorButton()
		colorbutton_highlight.set_color_from_string(self.highlight_color)
		label_highlight_color.set_mnemonic_widget(colorbutton_highlight)

		label_highlight_background = widget.Label(_("Highlight Background"))
		colorbutton_highlight_background = widget.ColorButton()
		colorbutton_highlight_background.set_color_from_string(self.background_highlight_color)
		label_highlight_background.set_mnemonic_widget(colorbutton_highlight_background)
		
		label_speech_module = widget.Label(_("Speech-Module"))
		combobox_speech_module = widget.ComboBox()
		for item in speech.Speech().list_output_modules():
			combobox_speech_module.add_item(item)
		combobox_speech_module.connect_change_callback_function(change_speech_module)
		label_speech_module.set_mnemonic_widget(combobox_speech_module)		

		label_speech_language = widget.Label(_("Speech-Language"))
		combobox_speech_language = widget.ComboBox()
		combobox_speech_module.set_active(self.speech_module)
		combobox_speech_language.set_active(self.speech_language)
		label_speech_language.set_mnemonic_widget(combobox_speech_language)

		label_speech_rate = widget.Label(_("Speech-Rate"))
		spin_speech_rate = widget.SpinButton(self.speech_rate,-100,100,1,10,0)
		label_speech_rate.set_mnemonic_widget(spin_speech_rate)
		label_speech_volume = widget.Label(_("Speech-Volume"))
		spin_speech_volume = widget.SpinButton(self.speech_volume,-100,100,1,10,0)
		label_speech_volume.set_mnemonic_widget(spin_speech_volume)
		label_speech_pitch = widget.Label(_("Speech-Pitch"))
		spin_speech_pitch = widget.SpinButton(self.speech_pitch,-100,100,1,10,0)
		label_speech_pitch.set_mnemonic_widget(spin_speech_pitch)

		
		grid_general = containers.Grid()
		grid_general.add_widgets(
			[(label_font,1,1),(fontbutton_font,1,1),containers.Grid.NEW_ROW,								  
			(label_font_color,1,1),(colorbutton_font,1,1),containers.Grid.NEW_ROW,								  
			(label_background_color,1,1),(colorbutton_background,1,1),containers.Grid.NEW_ROW,								  
			(label_highlight_font,1,1),(fontbutton_highlight_font,1,1),containers.Grid.NEW_ROW,								  
			(label_highlight_color,1,1),(colorbutton_highlight,1,1),containers.Grid.NEW_ROW,							  
			(label_highlight_background,1,1),(colorbutton_highlight_background,1,1),containers.Grid.NEW_ROW,
			(label_speech_module,1,1),(combobox_speech_module,1,1),containers.Grid.NEW_ROW,
			(label_speech_language,1,1),(combobox_speech_language,1,1),containers.Grid.NEW_ROW,
			(label_speech_rate,1,1),(spin_speech_rate,1,1),containers.Grid.NEW_ROW,
			(label_speech_pitch,1,1),(spin_speech_pitch,1,1),containers.Grid.NEW_ROW,
			(label_speech_volume,1,1),(spin_speech_volume,1,1)])
		notebook.add_page(_("General"),grid_general)
		grid_general.show_all()

		#RECOGNITION - PAGE ########
		#Engine		
		label_engine = widget.Label(_("Engine"))
		combobox_engine = widget.ComboBox()
		combobox_engine.connect_change_callback_function(change_engine)
		label_engine.set_mnemonic_widget(combobox_engine)
		
		for item in self.available_ocr_engine_list:
			combobox_engine.add_item(item[0])		
		
		#Language
		label_language = widget.Label(_("Language"))		
		combobox_language = widget.ComboBox()
		label_language.set_mnemonic_widget(combobox_language)
		
		#setting current engine - This can't be done before creating language combobox
		combobox_engine.set_active(self.ocr_engine)
		combobox_language.set_active(self.language)

		#Run text cleaner
		checkbutton_run_text_cleaner = widget.CheckButton(_("Run Text Cleaner"))
		checkbutton_run_text_cleaner.set_active(self.run_text_cleaner)
		
		#insert_position
		label_insert_position = widget.Label(_("Insert Position"))
		combobox_insert_position = widget.ComboBox()
		combobox_insert_position.add_item(_("Start"))
		combobox_insert_position.add_item(_("Cursor"))
		combobox_insert_position.add_item(_("End"))
		combobox_insert_position.set_active(self.insert_position)
		label_insert_position.set_mnemonic_widget(combobox_insert_position)
		
		#Seperator
		seperator_1 = widget.Separator()

		#Mode of Rotation
		label_mode_of_rotation = widget.Label(_("Mode Of Rotation"))
		combobox_mode_of_rotation = widget.ComboBox()
		combobox_mode_of_rotation.add_item(_("Full Automatic"))
		combobox_mode_of_rotation.add_item(_("Partial Automatic"))
		combobox_mode_of_rotation.add_item(_("Manual"))
		combobox_mode_of_rotation.connect_change_callback_function(change_mode_of_rotation)
		label_mode_of_rotation.set_mnemonic_widget(combobox_mode_of_rotation)
		
		#Angle
		self.label_angle = widget.Label(_("Angle"))		
		combobox_angle = widget.ComboBox()
		combobox_angle.add_item(_("00"))
		combobox_angle.add_item(_("90"))
		combobox_angle.add_item(_("180"))
		combobox_angle.add_item(_("270"))
		self.label_angle.set_mnemonic_widget(combobox_angle)		

		#Seperator 2
		seperator_2 = widget.Separator()
		
		#Give page number
		checkbutton_give_page_number = widget.CheckButton(_("Give Page Number"))
		checkbutton_give_page_number.set_active(self.give_page_number)

		#Page-Numbering
		label_numbering_type = widget.Label(_("Page Numbering Type"))				
		combobox_numbering_type = widget.ComboBox()
		combobox_numbering_type.add_item(_("Single Page"))
		combobox_numbering_type.add_item(_("Double Page"))
		combobox_numbering_type.set_active(self.page_numbering_type)
		label_numbering_type.set_mnemonic_widget(combobox_numbering_type)
		
		#Starting Page Number
		label_starting_page_number = widget.Label(_("Starting Page Number"))
		spin_starting_page_number = widget.SpinButton(0,0,100000,1,5,0)
		spin_starting_page_number.set_value(self.starting_page_number)
		label_starting_page_number.set_mnemonic_widget(spin_starting_page_number)
		
		grid_recognition = containers.Grid()
		grid_recognition.add_widgets([
			(label_engine,1,1),(combobox_engine,1,1),containers.Grid.NEW_ROW,
			(label_language,1,1),(combobox_language,1,1),containers.Grid.NEW_ROW,
			(checkbutton_run_text_cleaner,1,1),containers.Grid.NEW_ROW,
			(label_insert_position,1,1),(combobox_insert_position,1,1),containers.Grid.NEW_ROW,
			(seperator_1,2,1),containers.Grid.NEW_ROW,
			(label_mode_of_rotation,1,1),(combobox_mode_of_rotation,1,1),containers.Grid.NEW_ROW,
			(self.label_angle,1,1),(combobox_angle,1,1),containers.Grid.NEW_ROW,
			(seperator_2,2,1),containers.Grid.NEW_ROW,
			(checkbutton_give_page_number,1,1),containers.Grid.NEW_ROW,
			(label_numbering_type,1,1),	(combobox_numbering_type,1,1),containers.Grid.NEW_ROW,
			(label_starting_page_number,1,1),
			(spin_starting_page_number,1,1)])
		notebook.add_page(_("Recognition"),grid_recognition)
		grid_recognition.show_all()

		#setting current mode of rotation - This can't be done before creating angle combobox
		#also it should be here because the show_all function of grid will make angle combobox show again
		combobox_mode_of_rotation.set_active(self.mode_of_rotation)
		combobox_angle.set_active(self.rotation_angle)
		
		
		#SCANNING - PAGE ##############
		label_resolution = widget.Label(_("Resolution"))
		spin_resolution = widget.SpinButton(300,100,1200,1,5,0)
		spin_resolution.set_value(self.scan_resolution)
		label_resolution.set_mnemonic_widget(spin_resolution)

		label_brightness = widget.Label(_("Brightness"))
		spin_brightness = widget.SpinButton(50,0,100,1,5,0)
		spin_brightness.set_value(self.scan_brightness)
		label_brightness.set_mnemonic_widget(spin_brightness)

		label_scan_area = widget.Label(_("Scan Area"))
		combobox_scan_area = widget.ComboBox()
		label_scan_area.set_mnemonic_widget(combobox_scan_area)
		combobox_scan_area.add_item(_("Full Scan Area"))
		combobox_scan_area.add_item(_("Three Quarters"))
		combobox_scan_area.add_item(_("Two Quarters"))
		combobox_scan_area.add_item(_("One Quarters"))
		combobox_scan_area.set_active(self.scan_area)

		label_scan_driver = widget.Label(_("Driver"))
		combobox_scan_driver = widget.ComboBox()
		label_scan_driver.set_mnemonic_widget(combobox_scan_driver)		
		for item in self.available_scanner_drivers:
			combobox_scan_driver.add_item(item)
		combobox_scan_driver.set_active(self.scan_driver)

		sparator_3 = widget.Separator()
		
		label_number_of_pages_to_scan = widget.Label(_("Number of Pages to Scan"))
		spin_number_of_pages_to_scan = widget.SpinButton(10,2,100,1,5,0)
		spin_number_of_pages_to_scan.set_value(self.number_of_pages_to_scan)
		label_number_of_pages_to_scan.set_mnemonic_widget(spin_number_of_pages_to_scan)

		label_time_bitween_repeted_scanning = widget.Label(_("Time Bitween Repeted Scanning"))
		spin_time_bitween_repeted_scanning = widget.SpinButton(0,0,30,1,5,0)
		spin_time_bitween_repeted_scanning.set_value(self.time_between_repeated_scanning)
		label_time_bitween_repeted_scanning.set_mnemonic_widget(spin_time_bitween_repeted_scanning)

		sparator_4 = widget.Separator()

		checkbutton_scan_mode_switching = widget.CheckButton(_("Change to binary or lineart if possible"))
		checkbutton_scan_mode_switching.set_active(self.scanner_mode_switching)

		checkbutton_scanner_cache_calibration = widget.CheckButton(_("Cache Calibration"))
		checkbutton_scanner_cache_calibration.set_active(self.scanner_cache_calibration)
		
		grid_scanning = containers.Grid()
		grid_scanning.add_widgets([
			(label_resolution,1,1),(spin_resolution,1,1),containers.Grid.NEW_ROW,
			(label_brightness,1,1),(spin_brightness,1,1),containers.Grid.NEW_ROW,
			(label_scan_area,1,1),(combobox_scan_area,1,1),containers.Grid.NEW_ROW,
			(label_scan_driver,1,1),(combobox_scan_driver,1,1),containers.Grid.NEW_ROW,
			(sparator_3,2,1),containers.Grid.NEW_ROW,
			(label_number_of_pages_to_scan,1,1),(spin_number_of_pages_to_scan,1,1),containers.Grid.NEW_ROW,
			(label_time_bitween_repeted_scanning,1,1),(spin_time_bitween_repeted_scanning,1,1),containers.Grid.NEW_ROW,
			(sparator_4,2,1),containers.Grid.NEW_ROW,
			(checkbutton_scan_mode_switching,2,1),containers.Grid.NEW_ROW,
			(checkbutton_scanner_cache_calibration,2,1)])
		
		notebook.add_page(_("Scanning"),grid_scanning)
		grid_scanning.show_all()
		
		#Setting page
		notebook.set_current_page(page)
		
		dlg = dialog.Dialog(_("Lios Preferences"),(_("Apply"),dialog.Dialog.BUTTON_ID_1,_("Close"),dialog.Dialog.BUTTON_ID_2))
		dlg.add_widget(notebook)
		if (dlg.run()==True):
			self.font=fontbutton_font.get_font_name();
			self.font_color=colorbutton_font.get_color_as_string()			
			self.background_color=colorbutton_background.get_color_as_string()
			
			self.highlight_font=fontbutton_highlight_font.get_font_name();
			self.highlight_color=colorbutton_highlight.get_color_as_string()
			self.background_highlight_color=colorbutton_highlight_background.get_color_as_string()
			
			self.ocr_engine=combobox_engine.get_active()
			self.language=combobox_language.get_active()
			self.speech_module=combobox_speech_module.get_active()
			self.speech_language=combobox_speech_language.get_active()
			self.speech_rate=spin_speech_rate.get_value()
			self.speech_pitch=spin_speech_pitch.get_value()
			self.speech_volume=spin_speech_volume.get_value()
			self.insert_position=combobox_insert_position.get_active();
			self.mode_of_rotation=combobox_mode_of_rotation.get_active()
			self.rotation_angle = combobox_angle.get_active()
			self.give_page_number=int(checkbutton_give_page_number.get_active())
			self.page_numbering_type=combobox_numbering_type.get_active();
			self.starting_page_number=spin_starting_page_number.get_value_as_int();
			
			self.scan_resolution = spin_resolution.get_value_as_int();
			self.scan_brightness=spin_brightness.get_value_as_int();
			self.scan_area=combobox_scan_area.get_active();
			self.scan_driver=combobox_scan_driver.get_active();
			self.number_of_pages_to_scan=spin_number_of_pages_to_scan.get_value_as_int();
			self.time_between_repeated_scanning=spin_time_bitween_repeted_scanning.get_value_as_int();
			self.scanner_mode_switching=int(checkbutton_scan_mode_switching.get_active())
			self.run_text_cleaner=int(checkbutton_run_text_cleaner.get_active())
			self.scanner_cache_calibration=int(checkbutton_scanner_cache_calibration.get_active())
			
			dlg.destroy()
			return True
		dlg.destroy()
		return False

	
if "__main__" == __name__:
	a = lios_preferences()
	a.set_avalable_ocr_engines([])
	a.set_avalable_scanner_drivers([])
	a.open_configure_dialog()
	a.open_configure_dialog()
	
	loop.start_main_loop()
		

	




#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk


class Dialog(Gtk.Dialog):
	BUTTON_ID_1 = 1
	BUTTON_ID_2 = 2
	BUTTON_ID_3 = 3
	def __init__(self,title,buttons):
		super(Dialog,self).__init__(title,None,True,buttons)
	
	def add_widget(self,widget):
		box = self.get_content_area();
		box.add(widget)
	
	def add_widget_with_label(self,widget,label_text):
		new_box = Gtk.Box()
		label = Gtk.Label(label_text)
		label.set_mnemonic_widget(widget)
		new_box.pack_start(label, True, True, 0)
		new_box.pack_start(widget, True, True, 0)
		box = self.get_content_area();
		box.add(new_box)
		box.show_all()		

	def connect_configure_event_handler(self,function):
		self.connect("configure-event",function)
		
	#show_all()
	#run()



#!/usr/bin/python3 

###########################################################################
#    Lios - Linux-Intelligent-Ocr-Solution
#    Copyright (C) 2015-2016 Nalin.x.Linux GPL-3
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

from gi.repository import Gtk
from gi.repository import GLib
from gi.repository import Gdk
from gi.repository import Atk
from lios.ui.gtk import icon		


class Entry(Gtk.Entry):
	def __init__(self):
		super(Entry,self).__init__()
		
	def connect_change_handler(self,function):
		self.connect("changed",function)

	def connect_activate_function(self,function):
		self.connect("activate",function)

	#set_text()
	#get_text()		
		

class Label(Gtk.Label):
	def __init__(self,text):
		super(Label,self).__init__(text)

	#set_use_markup()

class Button(Gtk.Button):
	def __init__(self,label):
		super(Button,self).__init__(label)

	def connect_function(self,function):
		self.connect("clicked",function)

class IconButton(Gtk.Button):
	def __init__(self,label):
		super(IconButton,self).__init__()
		image = Gtk.Image()
		image.set_from_icon_name(icon.stock_icon_dict[label],8)
		self.set_image(image)
		self.set_hexpand(False)
		self.set_vexpand(False)

	def connect_function(self,function):
		self.connect("clicked",function)

class SpinButton(Gtk.SpinButton):
	def __init__(self,value=1,lower=0,upper=100,step_incr=1,page_incr=5,page_size=0):
		super(SpinButton,self).__init__()
		adj = Gtk.Adjustment(value, lower, upper, step_incr, page_incr, page_size)
		self.set_adjustment(adj)
		self.set_value(value)

	def connect_function(self,function):
		self.connect("clicked",function)
		
	#def set_value(self,value):
	#	self.set_value(value)
	
	def get_value(self):
		return self.get_value_as_int()

class ComboBox(Gtk.ComboBoxText):
	def __init__(self):
		super(ComboBox,self).__init__()
		model = Gtk.ListStore(str)
		self.set_model(model)
		renderer_text = Gtk.CellRendererText()
		self.pack_start(renderer_text, True)

	def add_item(self,item):
		model = self.get_model()
		model.append([item])
	def connect_change_callback_function(self,function):
		self.connect("changed",function);
	def clear(self):
		model = self.get_model()
		model.clear()
	#set_active


class ListView(Gtk.TreeView):
	def __init__(self,title):
		super(ListView,self).__init__()
		model = Gtk.ListStore(str)
		self.set_model(model)
		
		renderer = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn(title, renderer, text=0)
		self.append_column(column)
	
	def add_item(self,item):
		model = self.get_model()
		model.append([item])
	
	def get_selected_item(self):
		tree_selection =  self.get_selection()
		model,tree_iter = tree_selection.get_selected()
		return model.get_value(tree_iter,0)

	def get_selected_item_index(self):
		tree_selection =  self.get_selection()
		model,tree_iter = tree_selection.get_selected()
		i = 0;
		for item in model:
			if item[0] == model[tree_iter][0]:
				return i;
			i = i + 1;
		return -1;

	def remove_selected_item(self):
		tree_selection =  self.get_selection()
		model,tree_iter = tree_selection.get_selected()
		model.remove(tree_iter)

	
	def clear(self):
		model = self.get_model()
		model.clear()
		
	def connect_on_select_callback(self,function):
		self.connect("row-activated",function)

class ColorButton(Gtk.ColorButton):
	def __init__(self):
		super(ColorButton,self).__init__()
	
	def set_color_from_string(self,color):
		self.set_color(Gdk.color_parse(color))
	
	def get_color_as_string(self):
		return self.get_color().to_string();
		
		

class FontButton(Gtk.FontButton):
	def __init__(self):
		super(FontButton,self).__init__()
	def connect_function(self,function):
		self.connect("font-set",function)

	
	#set_font(font desc)
	#get_font_name()

class Separator(Gtk.HSeparator):
	def __init__(self):
		super(Separator,self).__init__()

class CheckButton(Gtk.CheckButton):
	def __int__(self,label):
		super(CheckButton,self).__int__()
		self.set_label(label)	

	def connect_handler_function(self,function):
		self.connect("clicked",function)

class ProgressBar(Gtk.ProgressBar):
	def __init__(self):
		super(ProgressBar,self).__init__()
		self.activity_mode = True
		a = GLib.timeout_add(20, self.progressbar_timeout, None)

	def progressbar_timeout(self, user_data):
		if self.activity_mode:
			self.pulse()
		#else:
		#	new_value = self.get_fraction() + 0.01
		#	if new_value > 1:
		#		new_value = 0
		#	self.set_fraction(new_value)
		return True	

	def set_pulse_mode(self,mode):
		self.activity_mode = mode;

class Statusbar(Gtk.Frame):
	def __init__(self):
		super(Statusbar,self).__init__()
		self.label = Gtk.Label()
		frame_inner = Gtk.Frame()
		frame_inner.add(self.label)
		atk_ob1 = frame_inner.get_accessible()
		atk_ob1.set_role(Atk.Role.NOTIFICATION)
		
		self.add(frame_inner)
		atk_ob = self.get_accessible()
		atk_ob.set_role(Atk.Role.STATUSBAR)
		
		
	def set_text(self,text):
		self.label.set_text(text)
		child = self.get_children()[0]
		atk_ob = child.get_accessible()
		atk_ob.notify_state_change(Atk.StateType.SHOWING,True);
	
	def set_line_wrap(self,val):
		self.label.set_line_wrap(val)
